@@ -3,17 +3,17 @@
 
 #include "amarok.h"
 #include "amarokconfig.h"
+#include "browserToolBar.h"
 #include "clicklineedit.h"
+#include "debug.h"
 #include "collectiondb.h"
 #include "config.h"
 #include "coverfetcher.h"
 #include "covermanager.h"
-#include "debug.h"
 
 #include <qdesktopwidget.h>  //ctor: desktop size
 #include <qfile.h>
 #include <qfontmetrics.h>    //paintItem()
-#include <qhbox.h>
 #include <qimage.h>
 #include <qlabel.h>
 #include <qlayout.h>
@@ -25,6 +25,7 @@
 #include <qprogressdialog.h>
 #include <qrect.h>
 #include <qstringlist.h>
+#include <qtooltip.h>
 #include <qtimer.h>    //search filter timer
 #include <qtooltip.h>
 #include <qvbox.h>
@@ -49,8 +50,12 @@
 #include <ktoolbarbutton.h>    //clear filter button
 #include <kurl.h>
 #include <kurldrag.h>
+#include <kwin.h>
 
 
+//Tooltips
+#define escapeHTMLAttr(s) QString(s).replace( "%", "%25" ).replace( "'", "%27" ).replace( "#", "%23" ).replace( "?", "%3F" )
+
 static QString artistToSelectInInitFunction;
 CoverManager *CoverManager::s_instance = 0;
 
@@ -69,9 +74,9 @@ CoverManager::CoverManager()
 
     // Sets caption and icon correctly (needed e.g. for GNOME)
     kapp->setTopWidget( this );
-    this->setCaption( kapp->makeStdCaption( i18n("Cover Manager") ) );
-    this->setWFlags( WDestructiveClose );
-    this->setMargin( 4 );
+    setCaption( kapp->makeStdCaption( i18n("Cover Manager") ) );
+    setWFlags( WDestructiveClose );
+    setMargin( 4 );
 
     //artist listview
     m_artistView = new KListView( this );
@@ -88,6 +93,14 @@ CoverManager::CoverManager()
         item = new KListViewItem( m_artistView, item, *it );
         item->setPixmap( 0, SmallIcon("personal") );
     }
+    QueryBuilder qb;
+    qb.addReturnValue( QueryBuilder::tabAlbum, QueryBuilder::valName );
+    qb.setOptions( QueryBuilder::optOnlyCompilations | QueryBuilder::optRemoveDuplicates );
+    qb.setLimit( 0, 1 );
+    if ( qb.run().count() ) {
+        item = new KListViewItem( m_artistView, item, i18n( "Various Artists" ) );
+        item->setPixmap( 0, SmallIcon("personal") );
+    }
 
     QVBox *vbox = new QVBox( this );
     QHBox *hbox = new QHBox( vbox );
@@ -97,11 +110,12 @@ CoverManager::CoverManager()
 
     { //<Search LineEdit>
         QHBox *searchBox = new QHBox( hbox );
-
-        KToolBarButton *button = new KToolBarButton( "locationbar_erase", 0, searchBox );
-        m_searchEdit = new ClickLineEdit( i18n( "Filter here..." ), searchBox );
+        KToolBar* searchToolBar = new Browser::ToolBar( searchBox );
+        KToolBarButton *button = new KToolBarButton( "locationbar_erase", 0, searchToolBar );
+        m_searchEdit = new ClickLineEdit( i18n( "Filter here..." ), searchToolBar );
         m_searchEdit->setFrame( QFrame::Sunken );
 
+        searchToolBar->setStretchableWidget( m_searchEdit );
         connect( button, SIGNAL(clicked()), m_searchEdit, SLOT(clear()) );
 
         QToolTip::add( button, i18n( "Clear filter" ) );
@@ -176,16 +190,24 @@ CoverManager::CoverManager()
 
 
     // signals and slots connections
-    connect( m_artistView, SIGNAL(selectionChanged( QListViewItem* ) ), SLOT(slotArtistSelected( QListViewItem* )) );
-    connect( m_coverView, SIGNAL(rightButtonPressed( QIconViewItem*, const QPoint& )), SLOT(showCoverMenu( QIconViewItem*, const QPoint& )) );
-    connect( m_coverView, SIGNAL(executed( QIconViewItem* )), SLOT(coverItemExecuted( QIconViewItem* )) );
-    connect( m_timer, SIGNAL(timeout()), SLOT(slotSetFilter()) );
-    connect( m_searchEdit, SIGNAL(textChanged( const QString& )), SLOT(slotSetFilterTimeout()) );
+    connect( m_artistView, SIGNAL(selectionChanged( QListViewItem* ) ),
+                           SLOT(slotArtistSelected( QListViewItem* )) );
+    connect( m_coverView,  SIGNAL(rightButtonPressed( QIconViewItem*, const QPoint& )),
+                           SLOT(showCoverMenu( QIconViewItem*, const QPoint& )) );
+    connect( m_coverView,  SIGNAL(executed( QIconViewItem* )),
+                           SLOT(coverItemExecuted( QIconViewItem* )) );
+    connect( m_timer,      SIGNAL(timeout()),
+                           SLOT(slotSetFilter()) );
+    connect( m_searchEdit, SIGNAL(textChanged( const QString& )),
+                           SLOT(slotSetFilterTimeout()) );
 
     #ifdef AMAZON_SUPPORT
-    connect( CollectionDB::instance(), SIGNAL(coverFetched( const QString&, const QString& )), SLOT(coverFetched( const QString&, const QString& )) );
-    connect( CollectionDB::instance(), SIGNAL(coverRemoved( const QString&, const QString& )), SLOT(coverRemoved( const QString&, const QString& )) );
-    connect( CollectionDB::instance(), SIGNAL(coverFetcherError( const QString& )), SLOT(coverFetcherError()) );
+    connect( CollectionDB::instance(), SIGNAL(coverFetched( const QString&, const QString& )),
+                                       SLOT(coverFetched( const QString&, const QString& )) );
+    connect( CollectionDB::instance(), SIGNAL(coverRemoved( const QString&, const QString& )),
+                                       SLOT(coverRemoved( const QString&, const QString& )) );
+    connect( CollectionDB::instance(), SIGNAL(coverFetcherError( const QString& )),
+                                       SLOT(coverFetcherError()) );
     #endif
 
     m_currentView = AllAlbums;
@@ -232,7 +254,10 @@ void CoverManager::viewCover( const QString& artist, const QString& album, QWidg
     //QDialog means "escape" works as expected
     QDialog *dialog = new QDialog( parent, 0, false, WDestructiveClose | WType_TopLevel );
     kapp->setTopWidget( dialog );
-    dialog->setCaption( kapp->makeStdCaption( artist + " - " + album ) );
+    dialog->setCaption( kapp->makeStdCaption( artist + i18n(" - ") + album ) );
+
+    // Gives the window a small title bar
+    KWin::setType( dialog->winId(), NET::Utility );
 
     QPixmap pixmap( CollectionDB::instance()->albumImage( artist, album, 0 ) );
     QLabel* label = new QLabel( dialog );
@@ -243,6 +268,8 @@ void CoverManager::viewCover( const QString& artist, const QString& album, QWidg
 
     dialog->show();
 }
+
+
 QString CoverManager::amazonTld() //static
 {
     if (AmarokConfig::amazonLocale() == "us")
@@ -252,9 +279,10 @@ QString CoverManager::amazonTld() //static
             else if (AmarokConfig::amazonLocale() == "uk")
         return "co.uk";
             else
-        return AmarokConfig::amazonLocale(); 
+        return AmarokConfig::amazonLocale();
 }
 
+
 void CoverManager::fetchMissingCovers() //SLOT
 {
     #ifdef AMAZON_SUPPORT
@@ -326,7 +354,7 @@ void CoverManager::slotArtistSelected( QListViewItem *item ) //SLOT
 
     QProgressDialog progress( this, 0, true );
     progress.setLabelText( i18n("Loading Thumbnails...") );
-    progress.QDialog::setCaption( "..." );
+    progress.QDialog::setCaption( i18n("...") );
 
     //NOTE we MUST show the dialog, otherwise the closeEvents get processed
     // in the processEvents() calls below, GRUMBLE! Qt sux0rs
@@ -355,13 +383,13 @@ void CoverManager::slotArtistSelected( QListViewItem *item ) //SLOT
 
     albums = qb.run();
 
-    //also retrieve compilations when we're showing all items (first treenode)
-    if ( item == m_artistView->firstChild() )
+    //also retrieve compilations when we're showing all items (first treenode) or
+    //"Various Artists" (last treenode)
+    if ( item == m_artistView->firstChild() || item == m_artistView->lastChild() )
     {
         QStringList cl;
 
         qb.clear();
-        qb.addReturnValue( QueryBuilder::tabDummy, QueryBuilder::valDummy );
         qb.addReturnValue( QueryBuilder::tabAlbum,  QueryBuilder::valName );
 
         qb.excludeMatch( QueryBuilder::tabAlbum, i18n( "Unknown" ) );
@@ -370,8 +398,10 @@ void CoverManager::slotArtistSelected( QListViewItem *item ) //SLOT
         qb.setOptions( QueryBuilder::optOnlyCompilations );
         cl = qb.run();
 
-        for ( uint i = 0; i < cl.count(); i++ )
+        for ( uint i = 0; i < cl.count(); i++ ) {
+            albums.append( i18n( "Various Artists" ) );
             albums.append( cl[ i ] );
+        }
     }
 
     QApplication::restoreOverrideCursor();
@@ -474,7 +504,6 @@ void CoverManager::showCoverMenu( QIconViewItem *item, const QPoint &p ) //SLOT
     #undef item
 }
 
-
 void CoverManager::coverItemExecuted( QIconViewItem *item ) //SLOT
 {
     #define item static_cast<CoverViewItem*>(item)
@@ -506,7 +535,7 @@ void CoverManager::slotSetFilter() //SLOT
     m_coverView->setAutoArrange( false );
     for( QIconViewItem *item = m_coverItems.first(); item; item = m_coverItems.next() ) {
         CoverViewItem *coverItem = static_cast<CoverViewItem*>(item);
-        if( coverItem->album().contains( m_filter, FALSE ) || coverItem->artist().contains( m_filter, FALSE ) )
+        if( coverItem->album().contains( m_filter, false ) || coverItem->artist().contains( m_filter, false ) )
             m_coverView->insertItem( item, m_coverView->lastItem() );
     }
     m_coverView->setAutoArrange( true );
@@ -541,7 +570,7 @@ void CoverManager::changeView( int id  ) //SLOT
         bool show = false;
         CoverViewItem *coverItem = static_cast<CoverViewItem*>(item);
         if( !m_filter.isEmpty() ) {
-            if( !coverItem->album().contains( m_filter, FALSE ) && !coverItem->artist().contains( m_filter, FALSE ) )
+            if( !coverItem->album().contains( m_filter, false ) && !coverItem->artist().contains( m_filter, false ) )
                 continue;
         }
 
@@ -652,7 +681,7 @@ void CoverManager::setCustomSelectedCovers()
     //function assumes something is selected
     QPtrList<CoverViewItem> selected = selectedItems();
     CoverViewItem* first = selected.getFirst();
-    
+
     QString artist_id; artist_id.setNum( CollectionDB::instance()->artistID( first->artist() ) );
     QString album_id; album_id.setNum( CollectionDB::instance()->albumID( first->album() ) );
     QStringList values = CollectionDB::instance()->albumTracks( artist_id, album_id );
@@ -667,7 +696,7 @@ void CoverManager::setCustomSelectedCovers()
     if ( !file.isEmpty() ) {
         qApp->processEvents();    //it may takes a while so process pending events
         QString tmpFile;
-        QImage image = CollectionDB::fetchImage(file, tmpFile);        
+        QImage image = CollectionDB::fetchImage(file, tmpFile);
         for ( CoverViewItem* item = selected.first(); item; item = selected.next() ) {
             CollectionDB::instance()->setAlbumImage( item->artist(), item->album(), image );
             item->loadCover();
@@ -700,7 +729,7 @@ void CoverManager::deleteSelectedCovers()
                                   "Are you sure you want to delete these %n covers from the Collection?",
                                   selected.count() ),
                             QString::null,
-                            i18n("&Remove") );
+                            KStdGuiItem::del() );
 
     if ( button == KMessageBox::Continue ) {
         for ( CoverViewItem* item = selected.first(); item; item = selected.next() ) {
@@ -753,8 +782,8 @@ void CoverManager::updateStatusBar()
             if ( values.count() >= 2 )
             {
                 text = i18n( "Fetching cover for " );
-                if ( !values[0].isEmpty() ) text += values[0] + " - ";
-                text += values[1] + "...";
+		if ( !values[0].isEmpty() ) text += values[0] + i18n(" - ");
+		text += values[1] + i18n("...");
             }
         }
         else if( m_fetchingCovers ) {
@@ -762,7 +791,7 @@ void CoverManager::updateStatusBar()
             if( m_coversFetched )
                 text += i18n( "1 fetched", "%n fetched", m_coversFetched );
             if( m_coverErrors ) {
-                if( m_coversFetched ) text += " - ";
+		    if( m_coversFetched ) text += i18n(" - ");
                 text += i18n( "1 not found", "%n not found", m_coverErrors );
             }
             if( m_coversFetched + m_coverErrors == 0 )
@@ -804,6 +833,12 @@ void CoverManager::updateStatusBar()
     m_statusLabel->setText( text );
 }
 
+void CoverManager::setStatusText( QString text )
+{
+    m_oldStatusText = m_statusLabel->text();
+    m_statusLabel->setText( text );
+}
+
 //////////////////////////////////////////////////////////////////////
 //    CLASS CoverView
 /////////////////////////////////////////////////////////////////////
@@ -819,6 +854,13 @@ CoverView::CoverView( QWidget *parent, const char *name, WFlags f )
     arrangeItemsInGrid();
     setAutoArrange( true );
     setItemsMovable( false );
+
+    // as long as QIconView only shows tooltips when the cursor is over the
+    // icon (and not the text), we have to create our own tooltips
+    setShowToolTips( false );
+
+    connect( this, SIGNAL( onItem( QIconViewItem * ) ), SLOT( setStatusText( QIconViewItem * ) ) );
+    connect( this, SIGNAL( onViewport() ), CoverManager::instance(), SLOT( updateStatusBar() ) );
 }
 
 
@@ -845,11 +887,30 @@ QDragObject *CoverView::dragObject()
     return drag;
 }
 
+void CoverView::setStatusText( QIconViewItem *item )
+{
+    #define item static_cast<CoverViewItem *>( item )
+    if ( !item )
+        return;
+
+    bool sampler = false;
+    //compilations have valDummy for artist.  see QueryBuilder::addReturnValue(..) for explanation
+    //FIXME: Don't rely on other independent code, use an sql query
+    if( item->artist().isEmpty() ) sampler = true;
+
+    QString tipContent = i18n( "%1 - %2" ).arg( sampler ? i18n("Various Artists") : item->artist() )
+                                          .arg( item->album() );
+
+    CoverManager::instance()->setStatusText( tipContent );
+
+    #undef item
+}
+
 //////////////////////////////////////////////////////////////////////
 //    CLASS CoverViewItem
 /////////////////////////////////////////////////////////////////////
 
-CoverViewItem::CoverViewItem( QIconView *parent, QIconViewItem *after, QString artist, QString album )
+CoverViewItem::CoverViewItem( QIconView *parent, QIconViewItem *after, const QString &artist, const QString &album )
     : KIconViewItem( parent, after, album )
     , m_artist( artist )
     , m_album( album )
@@ -914,14 +975,14 @@ void CoverViewItem::paintItem(QPainter* p, const QColorGroup& cg)
     int nameWidth = fm.width( str );
     if( nameWidth > textRect().width() )
     {
-        QString nameJustify = "...";
+	    QString nameJustify = i18n("...");
         int i = 0;
         while ( fm.width( nameJustify + str[ i ] ) < textRect().width() )
             nameJustify += str[ i++ ];
         nameJustify.remove( 0, 3 );
         if ( nameJustify.isEmpty() )
             nameJustify = str.left( 1 );
-        nameJustify += "...";
+	nameJustify += i18n("...");
         str = nameJustify;
     }
     p->setPen( cg.text() );
@@ -944,8 +1005,8 @@ void CoverViewItem::dropped( QDropEvent *e, const QValueList<QIconDragItem> & )
        if( hasCover() ) {
            int button = KMessageBox::warningContinueCancel( iconView(),
                             i18n( "Are you sure you want to overwrite this cover?"),
-                            QString::null,
-                            i18n("&Overwrite Confirmation") );
+                            i18n("Overwrite Confirmation"),
+                            i18n("&Overwrite") );
            if( button == KMessageBox::Cancel )
                return;
        }
