@@ -14,11 +14,13 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   51 Franklin Steet, Fifth Floor, Boston, MA  02111-1307, USA.          *
  ***************************************************************************/
 
 #define DEBUG_PREFIX "Gst-Engine"
 
+#include "debug.h"
+
 #include "config/gstconfig.h"
 #include "equalizer/gstequalizer.h"
 #include "enginebase.h"
@@ -27,6 +29,7 @@
 
 #include <math.h>
 #include <unistd.h>
+#include <vector>
 
 #include <qfile.h>
 #include <qtimer.h>
@@ -39,10 +42,6 @@
 
 #include <gst/gst.h>
 
-//HACK Needed until debug.h gets fixed
-#define indent gst_indent
-#include "debug.h"
-
 #define RETURN_IF_PIPELINE_EMPTY if ( !m_pipelineFilled ) return;
 
 
@@ -59,15 +58,10 @@ AMAROK_EXPORT_PLUGIN( GstEngine )
 /////////////////////////////////////////////////////////////////////////////////////
 
 void
-GstEngine::eos_cb( GstElement* /*element*/, InputPipeline* input ) //static
+GstEngine::eos_cb( GstElement* /*element*/, gpointer ) //static
 {
     DEBUG_FUNC_INFO
 
-    // Ignore eos when gst error was raised
-    if ( !instance()->m_gst_error.isEmpty() ) return;
-
-    input->m_eos = true;
-
     //this is the Qt equivalent to an idle function: delay the call until all events are finished,
     //otherwise gst will crash horribly
     QTimer::singleShot( 0, instance(), SLOT( endOfStreamReached() ) );
@@ -75,11 +69,11 @@ GstEngine::eos_cb( GstElement* /*element*/, InputPipeline* input ) //static
 
 
 void
-GstEngine::newPad_cb( GstElement*, GstPad* pad, gboolean, InputPipeline* input ) //static
+GstEngine::newPad_cb( GstElement*, GstPad* pad, gboolean, gpointer ) //static
 {
     DEBUG_BLOCK
 
-    GstPad* const audiopad = gst_element_get_pad( input->audioconvert, "sink" );
+    GstPad* const audiopad = gst_element_get_pad( instance()->m_gst_audioconvert, "sink" );
 
     if ( GST_PAD_IS_LINKED( audiopad ) ) {
         debug() << "audiopad is already linked. Unlinking old pad." << endl;
@@ -88,10 +82,10 @@ GstEngine::newPad_cb( GstElement*, GstPad* pad, gboolean, InputPipeline* input )
 
     gst_pad_link( pad, audiopad );
 
-    gst_element_unlink( input->volume, instance()->m_gst_adder );
-    gst_element_link( input->volume, instance()->m_gst_adder );
+    if ( gst_element_get_parent( instance()->m_gst_audiobin ) == NULL )
+        gst_bin_add( GST_BIN( instance()->m_gst_thread ), instance()->m_gst_audiobin );
 
-    gst_bin_sync_children_state( GST_BIN( input->bin ) );
+    gst_bin_sync_children_state( GST_BIN( instance()->m_gst_thread ) );
 }
 
 
@@ -163,7 +157,7 @@ GstEngine::found_tag_cb( GstElement*, GstElement*, GstTagList* taglist, gpointer
 
 
 void
-GstEngine::outputError_cb( GstElement* /*element*/, GstElement* /*domain*/, GError* error, gchar* debug, gpointer /*data*/ ) //static
+GstEngine::pipelineError_cb( GstElement* /*element*/, GstElement* /*domain*/, GError* error, gchar* debug, gpointer /*data*/ ) //static
 {
     DEBUG_FUNC_INFO
 
@@ -171,20 +165,7 @@ GstEngine::outputError_cb( GstElement* /*element*/, GstElement* /*domain*/, GErr
     instance()->m_gst_debug = QString::fromAscii( debug );
 
     // Process error message in application thread
-    QTimer::singleShot( 0, instance(), SLOT( handleOutputError() ) );
-}
-
-
-void
-GstEngine::inputError_cb( GstElement* /*element*/, GstElement* /*domain*/, GError* error, gchar* debug, gpointer /*inputPipeline*/ ) //static
-{
-    DEBUG_FUNC_INFO
-
-    instance()->m_gst_error = QString::fromAscii( error->message );
-    instance()->m_gst_debug = QString::fromAscii( debug );
-
-    // Process error message in application thread
-    QTimer::singleShot( 0, instance(), SLOT( handleInputError() ) );
+    QTimer::singleShot( 0, instance(), SLOT( handlePipelineError() ) );
 }
 
 
@@ -212,24 +193,21 @@ GstEngine::shutdown_cb() //static
 
 GstEngine::GstEngine()
         : Engine::Base()
-        , m_currentInput( 0 )
         , m_gst_adapter( 0 )
         , m_streamBuf( new char[STREAMBUF_SIZE] )
         , m_streamBuffering( false )
         , m_transferJob( 0 )
         , m_pipelineFilled( false )
         , m_fadeValue( 0.0 )
-        , m_eosReached( false )
+        , m_equalizerEnabled( false )
         , m_shutdown( false )
 {
-    kdDebug() << k_funcinfo << endl;
+    DEBUG_FUNC_INFO
 
     addPluginProperty( "StreamingMode", "Signal" );
     addPluginProperty( "HasConfigure",  "true" );
-    addPluginProperty( "HasCrossfade",  "true" );
     addPluginProperty( "HasEqualizer",  "true" );
-
-    m_inputs.setAutoDelete( true );
+    addPluginProperty( "HasKIO",        "true" );
 }
 
 
@@ -239,7 +217,7 @@ GstEngine::~GstEngine()
     debug() << "bytes left in gst_adapter: " << gst_adapter_available( m_gst_adapter ) << endl;
 
     if ( m_pipelineFilled ) {
-        g_signal_connect( G_OBJECT( m_gst_outputThread ), "shutdown", G_CALLBACK( shutdown_cb ), m_gst_outputThread );
+        g_signal_connect( G_OBJECT( m_gst_thread ), "shutdown", G_CALLBACK( shutdown_cb ), m_gst_thread );
         destroyPipeline();
         // Wait for pipeline to shut down properly
         while ( !m_shutdown ) ::usleep( 20000 ); // 20 msec
@@ -289,11 +267,6 @@ GstEngine::init()
         return false;
     }
 
-    if ( !createPipeline() )
-        error() << "createPipeline() failed.\n";
-
-    startTimer( TIMER_INTERVAL );
-
     return true;
 }
 
@@ -303,9 +276,12 @@ GstEngine::canDecode( const KURL &url ) const
 {
     // TODO Consider using decodebin here as well
 
-    // We had some bug reports claiming that .mov files cause crashes in canDecode(),
+    // We had some bug reports claiming that video files cause crashes in canDecode(),
     // so don't try to decode them
-    if ( url.fileName().lower().endsWith( ".mov" ) ) return false;
+    if ( url.fileName().lower().endsWith( ".mov" ) ||
+         url.fileName().lower().endsWith( ".avi" ) ||
+         url.fileName().lower().endsWith( ".wmv" ) )
+        return false;
 
     int count = 0;
     m_canDecodeSuccess = false;
@@ -341,12 +317,12 @@ GstEngine::canDecode( const KURL &url ) const
 uint
 GstEngine::position() const
 {
-    if ( !m_currentInput ) return 0;
+    if ( !m_pipelineFilled ) return 0;
 
     GstFormat fmt = GST_FORMAT_TIME;
     // Value will hold the current time position in nanoseconds. Must be initialized!
     gint64 value = 0;
-    gst_element_query( m_currentInput->decodebin, GST_QUERY_POSITION, &fmt, &value );
+    gst_element_query( m_gst_decodebin, GST_QUERY_POSITION, &fmt, &value );
 
     return static_cast<uint>( ( value / GST_MSECOND ) ); // nanosec -> msec
 }
@@ -357,12 +333,12 @@ GstEngine::length() const
 {
     DEBUG_BLOCK
 
-    if ( !m_currentInput ) return 0;
+    if ( !m_pipelineFilled ) return 0;
 
     GstFormat fmt = GST_FORMAT_TIME;
-    // Value will hold the current time position in nanoseconds. Must be initialized!
+    // Value will hold the track length in nanoseconds. Must be initialized!
     gint64 value = 0;
-    gst_element_query( m_currentInput->decodebin, GST_QUERY_TOTAL, &fmt, &value );
+    gst_element_query( m_gst_decodebin, GST_QUERY_TOTAL, &fmt, &value );
 
     return static_cast<uint>( ( value / GST_MSECOND ) ); // nanosec -> msec
 }
@@ -372,16 +348,9 @@ Engine::State
 GstEngine::state() const
 {
     if ( !m_pipelineFilled )
-        return Engine::Empty;
+        return m_url.isEmpty() ? Engine::Empty : Engine::Idle;
 
-    // amaroK expects the engine to return Idle on EOS
-    if ( m_eosReached )
-        return Engine::Idle;
-
-    if ( !m_currentInput )
-        return Engine::Empty;
-
-    switch ( gst_element_get_state( m_currentInput->bin ) )
+    switch ( gst_element_get_state( m_gst_thread ) )
     {
         case GST_STATE_NULL:
             return Engine::Empty;
@@ -464,7 +433,7 @@ GstEngine::configure() const
     DEBUG_FUNC_INFO
 
     GstConfigDialog* dialog = new GstConfigDialog( this );
-    connect( dialog, SIGNAL( settingsSaved() ), SLOT( configChanged() ) );
+    connect( dialog, SIGNAL( settingsSaved() ), SLOT( stop() ) );
 
     return dialog;
 }
@@ -481,28 +450,24 @@ GstEngine::load( const KURL& url, bool stream )  //SLOT
     Engine::Base::load( url, stream );
     debug() << "Loading url: " << url.url() << endl;
 
-    // Make sure we have a functional output pipeline
-    if ( !m_pipelineFilled && !createPipeline() )
+    if ( !createPipeline() )
         return false;
 
-    InputPipeline* input = new InputPipeline();
-    if ( input->m_error ) {
-        delete input;
-        return false;
-    }
+    m_gst_thread = gst_thread_new( "thread" );
+    g_signal_connect( G_OBJECT( m_gst_thread ), "error", G_CALLBACK ( pipelineError_cb ), NULL );
 
     if ( url.isLocalFile() ) {
         // Use gst's filesrc element for local files, cause it's less overhead than KIO
-        if ( !( input->src = createElement( "filesrc", input->bin ) ) ) { delete input; return false; }
+        if ( !( m_gst_src = createElement( "filesrc", m_gst_thread ) ) ) { destroyPipeline(); return false; }
         // Set file path
-        gst_element_set( input->src, "location", static_cast<const char*>( QFile::encodeName( url.path() ) ), NULL );
+        gst_element_set( m_gst_src, "location", static_cast<const char*>( QFile::encodeName( url.path() ) ), NULL );
     }
     else {
         // Create our custom streamsrc element, which transports data into the pipeline
-        input->src = GST_ELEMENT( gst_streamsrc_new( m_streamBuf, &m_streamBufIndex, &m_streamBufStop, &m_streamBuffering ) );
-        gst_element_set( input->src, "buffer_min", STREAMBUF_MIN, NULL );
-        gst_bin_add ( GST_BIN ( input->bin ), input->src );
-        g_signal_connect( G_OBJECT( input->src ), "kio_resume", G_CALLBACK( kio_resume_cb ), input->bin );
+        m_gst_src = GST_ELEMENT( gst_streamsrc_new( m_streamBuf, &m_streamBufIndex, &m_streamBufStop, &m_streamBuffering ) );
+        gst_element_set( m_gst_src, "buffer_min", STREAMBUF_MIN, NULL );
+        gst_bin_add ( GST_BIN ( m_gst_thread ), m_gst_src );
+        g_signal_connect( G_OBJECT( m_gst_src ), "kio_resume", G_CALLBACK( kio_resume_cb ), m_gst_thread );
 
         m_streamBufIndex = 0;
         m_streamBufStop = false;
@@ -516,31 +481,17 @@ GstEngine::load( const KURL& url, bool stream )  //SLOT
         }
     }
 
-    // Link all elements. The link from decodebin to audioconvert will be made in the newPad-callback
-    gst_element_link( input->src, input->decodebin );
-    gst_element_link_many( input->audioconvert, input->audioscale, input->volume, NULL );
+    if ( !( m_gst_decodebin = createElement( "decodebin", m_gst_thread ) ) ) { destroyPipeline(); return false; }
+    g_signal_connect( G_OBJECT( m_gst_decodebin ), "new-decoded-pad", G_CALLBACK( newPad_cb ), NULL );
+    g_signal_connect( G_OBJECT( m_gst_decodebin ), "found-tag", G_CALLBACK( found_tag_cb ), NULL );
+    g_signal_connect( G_OBJECT( m_gst_decodebin ), "eos", G_CALLBACK( eos_cb ), NULL );
 
-    // Prepare bin for playing
-    gst_element_set_state( input->bin, GST_STATE_READY );
+    // Link elements. The link from decodebin to audioconvert will be made in the newPad-callback
+    gst_element_link( m_gst_src, m_gst_decodebin );
 
-    // Check if another track is already playing
-    if ( m_currentInput ) {
-        if ( m_xfadeLength ) {
-            m_currentInput->setState( InputPipeline::XFADE_OUT );
-            input->setState( InputPipeline::XFADE_IN );
-            gst_element_set( input->volume, "volume", 0.0, NULL );
-        }
-        else
-            destroyInput( m_currentInput );
-    }
-    else
-        if ( GstConfig::fadeinDuration() ) {
-            input->setState( InputPipeline::FADE_IN );
-            gst_element_set( input->volume, "volume", 0.0, NULL );
-    }
-
-    m_currentInput = input;
-    m_inputs.append( input );
+    setVolume( m_volume );
+    setEqualizerEnabled( m_equalizerEnabled );
+    if ( m_equalizerEnabled ) setEqualizerParameters( m_equalizerPreamp, m_equalizerGains );
 
     return true;
 }
@@ -551,31 +502,13 @@ GstEngine::play( uint offset )  //SLOT
 {
     DEBUG_BLOCK
 
-    m_eosReached = false;
-    if ( !m_currentInput ) return false;
-
-    // We must pause the queue before changing the state of the input thread, else the scheduler
-    // freaks out. This is due to a bug in the queue element; will likely be fixed in gst-0.8.10.
-    // @see http://bugzilla.gnome.org/show_bug.cgi?id=166250
-    gst_element_set_state( m_gst_queue, GST_STATE_PAUSED );
-
-    if ( !gst_element_set_state( m_gst_inputThread, GST_STATE_PAUSED ) )
-        warning() << "Could not set input thread to PAUSED.\n";
-
-    gst_element_set_state( m_gst_queue, GST_STATE_PLAYING );
-
-    // Put input bin into input thread
-    gst_bin_add( GST_BIN( m_gst_inputThread ), m_currentInput->bin );
-
     // Try to play input pipeline; if fails, destroy input bin
-    if ( !gst_element_set_state( m_gst_inputThread, GST_STATE_PLAYING ) ) {
-        warning() << "Could not set input thread to PLAYING.\n";
-        destroyInput( m_currentInput );
+    if ( !gst_element_set_state( m_gst_thread, GST_STATE_PLAYING ) ) {
+        warning() << "Could not set thread to PLAYING.\n";
+        destroyPipeline();
         return false;
     }
 
-    g_signal_connect( G_OBJECT( m_currentInput->bin ), "error", G_CALLBACK ( inputError_cb ), m_currentInput );
-
     // If "Resume playback on start" is enabled, we must seek to the last position
     if ( offset ) seek( offset );
 
@@ -589,17 +522,22 @@ GstEngine::stop()  //SLOT
 {
     DEBUG_BLOCK
 
+    m_url = KURL(); // To ensure we return Empty from state()
+
+    if ( m_pipelineFilled )
+    {
+        // Is a fade running?
+        if ( m_fadeValue == 0.0 ) {
+            // Not fading --> start fade now
+            m_fadeValue = 1.0;
+            startTimer( TIMER_INTERVAL );
+        }
+        else
+            // Fading --> stop playback
+            destroyPipeline();
+    }
+
     emit stateChanged( Engine::Empty );
-    m_eosReached = false;
-
-    RETURN_IF_PIPELINE_EMPTY
-    if ( !m_currentInput ) return;
-
-    // When engine is in pause mode, don't fade but destroy right away
-    if ( state() == Engine::Paused )
-        destroyInput( m_currentInput );
-    else
-        m_currentInput->setState( InputPipeline::FADE_OUT );
 }
 
 
@@ -609,12 +547,12 @@ GstEngine::pause()  //SLOT
     DEBUG_BLOCK
     RETURN_IF_PIPELINE_EMPTY
 
-    if ( GST_STATE( m_currentInput->bin ) == GST_STATE_PAUSED ) {
-        gst_element_set_state( m_currentInput->bin, GST_STATE_PLAYING );
+    if ( GST_STATE( m_gst_thread ) == GST_STATE_PAUSED ) {
+        gst_element_set_state( m_gst_thread, GST_STATE_PLAYING );
         emit stateChanged( Engine::Playing );
     }
     else {
-        gst_element_set_state( m_currentInput->bin, GST_STATE_PAUSED );
+        gst_element_set_state( m_gst_thread, GST_STATE_PAUSED );
         emit stateChanged( Engine::Paused );
     }
 }
@@ -654,6 +592,8 @@ GstEngine::newStreamData( char* buf, int size )  //SLOT
 void
 GstEngine::setEqualizerEnabled( bool enabled ) //SLOT
 {
+    m_equalizerEnabled = enabled;
+
     RETURN_IF_PIPELINE_EMPTY
 
     gst_element_set( m_gst_equalizer, "active", enabled, NULL );
@@ -663,6 +603,9 @@ GstEngine::setEqualizerEnabled( bool enabled ) //SLOT
 void
 GstEngine::setEqualizerParameters( int preamp, const QValueList<int>& bandGains ) //SLOT
 {
+    m_equalizerPreamp = preamp;
+    m_equalizerGains = bandGains;
+
     RETURN_IF_PIPELINE_EMPTY
 
     // BEGIN Preamp
@@ -670,11 +613,12 @@ GstEngine::setEqualizerParameters( int preamp, const QValueList<int>& bandGains
     // END
 
     // BEGIN Gains
-    m_equalizerGains.resize( bandGains.count() );
+    vector<int> gainsTemp;
+    gainsTemp.resize( bandGains.count() );
     for ( uint i = 0; i < bandGains.count(); i++ )
-        m_equalizerGains[i] = ( *bandGains.at( i ) + 100 ) / 2;
+        gainsTemp[i] = ( *bandGains.at( i ) + 100 ) / 2;
 
-    gst_element_set( m_gst_equalizer, "gain", &m_equalizerGains, NULL );
+    gst_element_set( m_gst_equalizer, "gain", &gainsTemp, NULL );
     // END
 }
 
@@ -684,100 +628,39 @@ GstEngine::setEqualizerParameters( int preamp, const QValueList<int>& bandGains
 /////////////////////////////////////////////////////////////////////////////////////
 
 void
-GstEngine::setVolumeSW( uint percent )  //SLOT
+GstEngine::setVolumeSW( uint percent )
 {
     RETURN_IF_PIPELINE_EMPTY
 
-    gst_element_set( m_gst_volume, "volume", (double) percent * 0.01, NULL );
+    double fade;
+    if ( m_fadeValue > 0.0 )
+        fade = 1.0 - log10( ( 1.0 - m_fadeValue ) * 9.0 + 1.0 );
+    else
+        fade = 1.0;
+
+    gst_element_set( m_gst_volume, "volume", (double) percent * fade * 0.01, NULL );
 }
 
 
 void GstEngine::timerEvent( QTimerEvent* )
 {
-    // Display stream buffer status
-    sendBufferStatus();
+    // *** Volume fading ***
 
-    // Fading transition management:
-    QPtrList<InputPipeline> destroyList;
-    InputPipeline* input;
-
-    // Iterate over all input pipelines
-    for ( uint i = 0; i < m_inputs.count(); i++ )
+    // Are we currently fading?
+    if ( m_fadeValue > 0.0 )
     {
-        input = m_inputs.at( i );
+        m_fadeValue -= ( GstConfig::fadeoutDuration() ) ?  1.0 / GstConfig::fadeoutDuration() * TIMER_INTERVAL : 1.0;
 
-        switch ( input->state() )
-        {
-            case InputPipeline::NO_FADE:
-                break;
-
-            case InputPipeline::FADE_IN:
-                input->m_fade -= ( GstConfig::fadeinDuration() ) ?  1.0 / GstConfig::fadeinDuration() * TIMER_INTERVAL : 1.0;
-
-                // Fade finished?
-                if ( input->m_fade < 0.0 ) {
-                    // Fade transition has finished, stop playback
-                    debug() << "Fade-in finished.\n";
-                    input->setState( InputPipeline::NO_FADE );
-                }
-                else {
-                    // Set new value for fadeout volume element
-                    const double value = 1.0 - log10( input->m_fade * 9.0 + 1.0 );
-//                     kdDebug() << "FADE_IN: " << value << endl;
-                    gst_element_set( input->volume, "volume", value, NULL );
-                }
-                break;
-
-            case InputPipeline::FADE_OUT:
-                input->m_fade -= ( GstConfig::fadeoutDuration() ) ?  1.0 / GstConfig::fadeoutDuration() * TIMER_INTERVAL : 1.0;
-
-                // Fade finished?
-                if ( input->m_fade < 0.0 )
-                    destroyList.append( input );
-                else {
-                    // Set new value for fadeout volume element
-                    const double value = 1.0 - log10( ( 1.0 - input->m_fade ) * 9.0 + 1.0 );
-//                     kdDebug() << "FADE_OUT: " << value << endl;
-                    gst_element_set( input->volume, "volume", value, NULL );
-                }
-                break;
-
-            case InputPipeline::XFADE_IN:
-                input->m_fade -= ( m_xfadeLength ) ?  1.0 / m_xfadeLength * TIMER_INTERVAL : 1.0;
-
-                // Fade finished?
-                if ( input->m_fade < 0.0 ) {
-                    // Fade transition has finished, stop playback
-                    debug() << "XFade-in finished.\n";
-                    input->setState( InputPipeline::NO_FADE );
-                }
-                else {
-                    // Set new value for fadeout volume element
-                    const double value = 1.0 - input->m_fade;
-//                     kdDebug() << "XFADE_IN: " << value << endl;
-                    gst_element_set( input->volume, "volume", value, NULL );
-                }
-                break;
-
-            case InputPipeline::XFADE_OUT:
-                input->m_fade -= ( m_xfadeLength ) ?  1.0 / m_xfadeLength * TIMER_INTERVAL : 1.0;
-
-                // Fade finished?
-                if ( input->m_fade < 0.0 )
-                    destroyList.append( input );
-                else {
-                    // Set new value for fadeout volume element
-                    const double value = 1.0 - log10( ( 1.0 - input->m_fade ) * 9.0 + 1.0 );
-//                     kdDebug() << "XFADE_OUT: " << value << endl;
-                    gst_element_set( input->volume, "volume", value, NULL );
-                }
-                break;
+        // Fade finished?
+        if ( m_fadeValue <= 0.0 ) {
+            // Fade transition has finished, stop playback
+            debug() << "[Gst-Engine] Fade-out finished.\n";
+            destroyPipeline();
         }
+        setVolume( volume() );
     }
-
-    // Destroy pipelines which are finished
-    for ( uint i = 0; i < destroyList.count(); i++ )
-        destroyInput( destroyList.at( i ) );
+    else
+        killTimers();
 }
 
 
@@ -786,7 +669,7 @@ void GstEngine::timerEvent( QTimerEvent* )
 /////////////////////////////////////////////////////////////////////////////////////
 
 void
-GstEngine::handleOutputError()  //SLOT
+GstEngine::handlePipelineError()  //SLOT
 {
     DEBUG_BLOCK
 
@@ -802,31 +685,6 @@ GstEngine::handleOutputError()  //SLOT
     emit statusText( text );
     error() << text << endl;
 
-    // Destroy all pipelines
-    destroyPipeline();
-
-    emit trackEnded();
-}
-
-
-void
-GstEngine::handleInputError()  //SLOT
-{
-    DEBUG_BLOCK
-
-    QString text = "[GStreamer Error] ";
-    text += m_gst_error;
-
-    if ( !m_gst_debug.isEmpty() ) {
-        text += " ** ";
-        text += m_gst_debug;
-    }
-    m_gst_error = QString();
-    error() << text << endl;
-    emit statusText( text );
-    error() << "Input-Pipeline has signaled an error. Destroying pipeline." << endl;
-
-    m_inputError = true;
     destroyPipeline();
 }
 
@@ -836,25 +694,8 @@ GstEngine::endOfStreamReached()  //SLOT
 {
     DEBUG_BLOCK
 
-    InputPipeline* input;
-
-    // Iterate over all input pipelines
-    for ( uint i = 0; i < m_inputs.count(); i++ ) {
-        input = m_inputs.at( i );
-        if ( input->m_eos ) {
-            kdDebug() << "An input pipeline has reached EOS, destroying.\n";
-
-            const bool fading = input->state() == InputPipeline::FADE_OUT ||
-                                input->state() == InputPipeline::XFADE_OUT;
-
-            destroyInput( input );
-
-            // Simulate Idle state on EOS, if we are not fading
-            if ( !m_currentInput ) m_eosReached = true;
-
-            if ( !fading ) emit trackEnded();
-        }
-    }
+    destroyPipeline();
+    emit trackEnded();
 }
 
 
@@ -912,18 +753,6 @@ GstEngine::errorNoOutput() //SLOT
 }
 
 
-void
-GstEngine::configChanged() //SLOT
-{
-    debug() << "Rebuilding output pipeline with new settings.\n";
-
-    // Stop playback and rebuild output pipeline, in order to apply new settings
-    createPipeline();
-
-    emit stateChanged( Engine::Empty );
-}
-
-
 /////////////////////////////////////////////////////////////////////////////////////
 // PRIVATE METHODS
 /////////////////////////////////////////////////////////////////////////////////////
@@ -977,7 +806,7 @@ GstEngine::getPluginList( const QCString& classname ) const
 
                     if ( g_strrstr ( factory->details.klass, classname ) ) {
                         name = g_strdup ( GST_OBJECT_NAME ( factory ) );
-                        /*if ( name != "artsdsink" )*/
+                        if ( name != "alsasink" )
                         results << name;
                     }
                 }
@@ -999,31 +828,18 @@ GstEngine::createPipeline()
 {
     DEBUG_BLOCK
 
-    if ( m_pipelineFilled )
-        destroyPipeline();
+    destroyPipeline();
 
-    if ( GstConfig::soundOutput().isEmpty() ) {
+    if ( GstConfig::soundOutput().isEmpty() || GstConfig::soundOutput() == "alsasink" ) {
         QTimer::singleShot( 0, this, SLOT( errorNoOutput() ) );
         return false;
     }
-    debug() << "Thread scheduling priority: " << GstConfig::threadPriority() << endl;
     debug() << "Sound output method: " << GstConfig::soundOutput() << endl;
     debug() << "CustomSoundDevice: " << ( GstConfig::useCustomSoundDevice() ? "true" : "false" ) << endl;
     debug() << "Sound Device: " << GstConfig::soundDevice() << endl;
     debug() << "CustomOutputParams: " << ( GstConfig::useCustomOutputParams() ? "true" : "false" ) << endl;
     debug() << "Output Params: " << GstConfig::outputParams() << endl;
 
-    m_gst_rootBin = gst_bin_new( "root_bin" );
-
-    //<input>
-    if ( !( m_gst_inputThread = createElement( "thread" ) ) ) { return false; }
-    if ( !( m_gst_adder = createElement( "adder", m_gst_inputThread ) ) ) { return false; }
-    //</input>
-
-    /* create a new pipeline (thread) to hold the elements */
-    if ( !( m_gst_outputThread = createElement( "thread" ) ) ) { return false; }
-    gst_element_set( m_gst_outputThread, "priority", GstConfig::threadPriority(), NULL );
-
     // Let gst construct the output element from a string
     QCString output  = GstConfig::soundOutput().latin1();
     if ( GstConfig::useCustomOutputParams() ) {
@@ -1035,45 +851,30 @@ GstEngine::createPipeline()
         QTimer::singleShot( 0, this, SLOT( errorNoOutput() ) );
         return false;
     }
-    gst_bin_add( GST_BIN( m_gst_outputThread ), m_gst_audiosink );
+
+    m_gst_audiobin = gst_bin_new( "audiobin" );
+    gst_bin_add( GST_BIN( m_gst_audiobin ), m_gst_audiosink );
 
     /* setting device property for AudioSink*/
     if ( GstConfig::useCustomSoundDevice() && !GstConfig::soundDevice().isEmpty() )
         gst_element_set( m_gst_audiosink, "device", GstConfig::soundDevice().latin1(), NULL );
 
-    if ( !( m_gst_queue = createElement( "queue", m_gst_outputThread ) ) ) { return false; }
     m_gst_equalizer = GST_ELEMENT( gst_equalizer_new() );
-    gst_bin_add( GST_BIN( m_gst_outputThread ), m_gst_equalizer );
-    if ( !( m_gst_identity = createElement( "identity", m_gst_outputThread ) ) ) { return false; }
-    if ( !( m_gst_volume = createElement( "volume", m_gst_outputThread ) ) ) { return false; }
-
-    // Put everything into the root bin
-    gst_bin_add_many( GST_BIN( m_gst_rootBin ), m_gst_inputThread, m_gst_outputThread, NULL );
-
-    // More buffers means less dropouts and higher latency
-    gst_element_set( m_gst_queue, "max-size-buffers", 60, NULL );
+    gst_bin_add( GST_BIN( m_gst_audiobin ), m_gst_equalizer );
+    if ( !( m_gst_audioconvert = createElement( "audioconvert", m_gst_audiobin ) ) ) { return false; }
+    if ( !( m_gst_identity = createElement( "identity", m_gst_audiobin ) ) ) { return false; }
+    if ( !( m_gst_volume = createElement( "volume", m_gst_audiobin ) ) ) { return false; }
+    if ( !( m_gst_audioscale = createElement( "audioscale", m_gst_audiobin ) ) ) { return false; }
 
     g_signal_connect( G_OBJECT( m_gst_identity ), "handoff", G_CALLBACK( handoff_cb ), NULL );
-    g_signal_connect ( G_OBJECT( m_gst_outputThread ), "error", G_CALLBACK ( outputError_cb ), NULL );
 
     /* link elements */
-    gst_element_link_many( m_gst_adder, m_gst_queue, m_gst_equalizer,  m_gst_identity, m_gst_volume, m_gst_audiosink, NULL );
+    gst_element_link_many( m_gst_audioconvert, m_gst_equalizer, m_gst_identity,
+                           m_gst_volume, m_gst_audioscale, m_gst_audiosink, NULL );
 
-    setVolume( m_volume );
-
-    if ( !gst_element_set_state( m_gst_inputThread, GST_STATE_READY ) ) {
-        error() << "Could not set inputThread to state READY!\n";
-        destroyPipeline();
-        return false;
-    }
-    if ( !gst_element_set_state( m_gst_outputThread, GST_STATE_PLAYING ) ) {
-        error() << "Could not set outputThread to state PLAYING!\n";
-        destroyPipeline();
-        return false;
-    }
+    gst_element_set_state( m_gst_audiobin, GST_STATE_PAUSED );
 
     m_pipelineFilled = true;
-    m_inputError = false;
     return true;
 }
 
@@ -1085,17 +886,15 @@ GstEngine::destroyPipeline()
 
     m_fadeValue = 0.0;
 
-    // Destroy all input pipelines
-    m_inputs.clear();
-
     // Clear the scope adapter
     m_mutexScope.lock();
     gst_adapter_clear( m_gst_adapter );
     m_mutexScope.unlock();
 
     if ( m_pipelineFilled ) {
-        debug() << "Unreffing root bin." << endl;
-        gst_object_unref( GST_OBJECT( m_gst_rootBin ) );
+        debug() << "Unreffing pipeline." << endl;
+        gst_element_set_state( m_gst_thread, GST_STATE_NULL );
+        gst_object_unref( GST_OBJECT( m_gst_thread ) );
 
         m_pipelineFilled = false;
     }
@@ -1108,25 +907,6 @@ GstEngine::destroyPipeline()
 }
 
 
-void
-GstEngine::destroyInput( InputPipeline* input )
-{
-    DEBUG_BLOCK
-
-    if ( input ) {
-        debug() << "Destroying input pipeline.\n";
-        // Destroy the pipeline
-        m_inputs.remove( input );
-    }
-
-    // Destroy KIO transmission job
-    if ( m_transferJob ) {
-        m_transferJob->kill();
-        m_transferJob = 0;
-    }
-}
-
-
 void
 GstEngine::sendBufferStatus()
 {
@@ -1137,102 +917,4 @@ GstEngine::sendBufferStatus()
 }
 
 
-/////////////////////////////////////////////////////////////////////////////////////
-// CLASS InputPipeline
-/////////////////////////////////////////////////////////////////////////////////////
-
-InputPipeline::InputPipeline()
-    : m_state( NO_FADE )
-    , m_fade( 0.0 )
-    , m_error( false )
-    , m_eos( false )
-{
-    DEBUG_BLOCK
-
-    /* create a new pipeline (thread) to hold the elements */
-    if ( !( bin = GstEngine::createElement( "bin" ) ) ) { goto error; }
-    if ( !( decodebin = GstEngine::createElement( "decodebin", bin ) ) ) { goto error; }
-    if ( !( audioconvert = GstEngine::createElement( "audioconvert", bin ) ) ) { goto error; }
-    if ( !( audioscale = GstEngine::createElement( "audioscale", bin ) ) ) { goto error; }
-    if ( !( volume = GstEngine::createElement( "volume", bin ) ) ) { goto error; }
-
-    g_signal_connect( G_OBJECT( decodebin ), "eos", G_CALLBACK( GstEngine::eos_cb ), this );
-    g_signal_connect( G_OBJECT( decodebin ), "new-decoded-pad", G_CALLBACK( GstEngine::newPad_cb ), this );
-    g_signal_connect( G_OBJECT( decodebin ), "found-tag", G_CALLBACK( GstEngine::found_tag_cb ), 0 );
-
-    return;
-
-error:
-    m_error = true;
-}
-
-
-InputPipeline::~InputPipeline()
-{
-    DEBUG_BLOCK
-
-    if ( GstEngine::instance()->m_currentInput == this )
-        GstEngine::instance()->m_currentInput = 0;
-
-    if ( GstEngine::instance()->m_inputError )
-        return;
-
-    debug() << "Destroying input bin.\n";
-
-    if ( gst_element_get_managing_bin( bin ) == GST_BIN( GstEngine::instance()->m_gst_inputThread ) )
-    {
-        // We must pause the queue before changing the state of the input thread, else the scheduler
-        // freaks out. This is due to a bug in the queue element; will likely be fixed in gst-0.8.10.
-        // @see http://bugzilla.gnome.org/show_bug.cgi?id=166250
-        gst_element_set_state( GstEngine::instance()->m_gst_queue, GST_STATE_PAUSED );
-
-        if ( !gst_element_set_state( GstEngine::instance()->m_gst_inputThread, GST_STATE_PAUSED ) )
-            warning() << "Could not set input thread to PAUSED.\n";
-
-        gst_element_set_state( GstEngine::instance()->m_gst_queue, GST_STATE_PLAYING );
-
-        gst_element_unlink( volume, GstEngine::instance()->m_gst_adder );
-
-        // Destroy bin
-        gst_element_set_state( bin, GST_STATE_NULL );
-        gst_bin_remove( GST_BIN( GstEngine::instance()->m_gst_inputThread ), bin );
-
-        if ( !gst_element_set_state( GstEngine::instance()->m_gst_inputThread, GST_STATE_PLAYING ) )
-            warning() << "Could not set input thread to PLAYING.\n";
-    }
-    else
-    {
-        debug() << "Bin is not in thread.\n";
-        gst_element_set_state( bin, GST_STATE_NULL );
-        gst_bin_remove( GST_BIN( GstEngine::instance()->m_gst_inputThread ), bin );
-    }
-}
-
-
-void
-InputPipeline::setState( State newState )
-{
-    if ( m_error ) return;
-
-    switch ( newState )
-    {
-        case NO_FADE:
-            m_fade = 0.0;
-            break;
-
-        default:
-            if ( m_state == NO_FADE )
-                m_fade = 1.0;
-    }
-
-    m_state = newState;
-}
-
-
-#undef gst_indent
-namespace Debug { QCString gst_indent; }
-
-
 #include "gstengine.moc"
-
-
