@@ -10,31 +10,33 @@
  ***************************************************************************/
 
 #include "xine-config.h"
+#include "xinecfg.h"
 #include "xine-engine.h"
-#include "xine-scope.h"
 
 AMAROK_EXPORT_PLUGIN( XineEngine )
 
-#define DEBUG_PREFIX "xine-engine"
-#define indent xine_indent
-
 #include <climits>
 #include <cmath>
 #include "debug.h"
+
 #include <klocale.h>
 #include <kmessagebox.h>
+#include <kstandarddirs.h>
+
 #include <qapplication.h>
 #include <qdir.h>
 
 extern "C"
 {
     #include <unistd.h>
+    #include "xine-scope.h"
 }
 
 #ifndef LLONG_MAX
 #define LLONG_MAX 9223372036854775807LL
 #endif
 
+
 //define this to use xine in a more standard way
 //#define XINE_SAFE_MODE
 
@@ -47,8 +49,10 @@ namespace Log
     static uint noSuitableBuffer = 0;
 };
 
-///returns the configuration we will use
-static inline QCString configPath() { return QFile::encodeName( QDir::homeDirPath() + "/.xine/config" ); }
+///returns the configuration we will use. there is no KInstance, so using this hacked up method.
+//static inline QCString configPath() { return QFile::encodeName(KStandardDirs().localkdedir() + KStandardDirs::kde_default("data") + "amarok/xine-config"); }
+static inline QCString configPath() { return QFile::encodeName(locate( "data", "amarok/") + "xine-config" ); }
+static Fader *s_fader = 0;
 
 
 XineEngine::XineEngine()
@@ -58,35 +62,43 @@ XineEngine::XineEngine()
         , m_audioPort( 0 )
         , m_eventQueue( 0 )
         , m_post( 0 )
+        , m_preamp( 1.0 )
+        , m_stopFader( false )
 {
     addPluginProperty( "StreamingMode", "NoStreaming" );
     addPluginProperty( "HasConfigure", "true" );
     addPluginProperty( "HasEqualizer", "true" );
-    //addPluginProperty( "HasCrossfade", "true" );
+    addPluginProperty( "HasCrossfade", "true" );
 }
 
 XineEngine::~XineEngine()
 {
-//     if( m_stream && xine_get_status( m_stream ) == XINE_STATUS_PLAY )
-//     {
-//         const int volume = xine_get_param( m_stream, XINE_PARAM_AUDIO_AMP_LEVEL );
-//         const double D = 300000 * std::pow( (double)volume, -0.4951 );
-//
-//         debug() << "Sleeping: " << D << ", " << volume << endl;
-//
-//         for( int v = volume - 1; v >= 1; v-- ) {
-//             xine_set_param( m_stream, XINE_PARAM_AUDIO_AMP_LEVEL, v );
-//
-//             const int sleep = int(D * (-log10( v + 1 ) + 2));
-//
-//             ::usleep( sleep );
-//
-//             debug() << v << ": " << sleep << "us\n";
-//         }
-//         xine_stop( m_stream );
-//     }
+    // Wait until the fader thread is done
+    if( s_fader ) {
+        m_stopFader = true;
+        s_fader->wait();
+        delete s_fader;
+    }
 
-    xine_config_save( m_xine, configPath() );
+    // NOTE The fadeout gets stuck when the EQ is active, so we skip it then
+    if( !m_equalizerEnabled && m_stream && xine_get_status( m_stream ) == XINE_STATUS_PLAY )
+    {
+        const int volume = xine_get_param( m_stream, XINE_PARAM_AUDIO_AMP_LEVEL );
+        const double D = 300000 * std::pow( (double)volume, -0.4951 );
+
+        debug() << "Sleeping: " << D << ", " << volume << endl;
+
+        for( int v = volume - 1; v >= 1; v-- ) {
+            xine_set_param( m_stream, XINE_PARAM_AUDIO_AMP_LEVEL, v );
+
+            const int sleep = int(D * (-log10( v + 1 ) + 2));
+
+            ::usleep( sleep );
+        }
+        xine_stop( m_stream );
+    }
+
+    if( m_xine )       xine_config_save( m_xine, configPath() );
 
     if( m_stream )     xine_close( m_stream );
     if( m_eventQueue ) xine_event_dispose_queue( m_eventQueue );
@@ -105,6 +117,8 @@ XineEngine::~XineEngine()
 bool
 XineEngine::init()
 {
+   DEBUG_BLOCK
+
    debug() << "'Bringing joy to small mexican gerbils, a few weeks at a time.'\n";
 
    m_xine = xine_new();
@@ -119,6 +133,7 @@ XineEngine::init()
    #endif
 
    xine_config_load( m_xine, configPath() );
+   debug() << "w00t" << configPath() << endl;
 
    xine_init( m_xine );
 
@@ -135,12 +150,13 @@ XineEngine::init()
 bool
 XineEngine::makeNewStream()
 {
+    m_currentAudioPlugin = XineCfg::outputPlugin();
    {
       ///this block is so we don't crash if we fail to allocate a thingy
       xine_stream_t     *stream;
       xine_audio_port_t *port;
 
-      port = xine_open_audio_driver( m_xine, "auto", NULL );
+      port = xine_open_audio_driver( m_xine, XineCfg::outputPlugin().local8Bit(), NULL );
       if( !port ) {
          //TODO make engine method that is the same but parents the dialog for us
          KMessageBox::error( 0, i18n("xine was unable to initialize any audio-drivers.") );
@@ -177,23 +193,19 @@ XineEngine::makeNewStream()
    return true;
 }
 
-
-static Fader *s_fader = 0;
-
 bool
 XineEngine::load( const KURL &url, bool isStream )
 {
-   if( XINE_VERSION == "1-rc6a" && url.protocol() == "http" ) {
-      emit infoMessage( i18n("Sorry xine 1-rc6a cannot play remote streams, please upgrade to 1-rc7") );
-      return false;
-   }
+    Engine::Base::load( url, isStream );
 
-   Engine::Base::load( url, isStream || url.protocol() == "http" );
+    if( m_xfadeLength > 0 && xine_get_status( m_stream ) == XINE_STATUS_PLAY )
+    {
+       s_fader = new Fader( this );
+    }
 
-//     if( false && m_xfadeLength > 0 && xine_get_status( m_stream ) == XINE_STATUS_PLAY )
-//     {
-//        s_fader = new Fader( this );
-//     }
+   // for users who stubbonly refuse to use DMIX or buy a good soundcard
+   // why doesn't xine do this? I cannot say.
+   xine_close( m_stream );
 
    if( xine_open( m_stream, QFile::encodeName( url.url() ) ) )
    {
@@ -220,10 +232,20 @@ XineEngine::play( uint offset )
     if( xine_play( m_stream, 0, offset ) )
     {
         if( s_fader )
-           s_fader->start();
+           s_fader->start( QThread::LowestPriority );
 
         emit stateChanged( Engine::Playing );
 
+        //HACK Emit two dummy bundles, because the context-browser expects them for
+        //     the stream metadata display.
+        if( isStream() ) {
+            Engine::SimpleMetaBundle bundle;
+            bundle.title = "dummy";
+            emit metaData( bundle );
+            bundle.title = i18n( "Radio Stream" );
+            emit metaData( bundle );
+        }
+
         return true;
     }
 
@@ -309,6 +331,9 @@ XineEngine::position() const
 uint
 XineEngine::length() const
 {
+    if( m_url.fileName().endsWith( ".ogg", false ) )
+        return 0;
+
     int pos;
     int time;
     int length = 0;
@@ -321,47 +346,65 @@ XineEngine::length() const
 void
 XineEngine::seek( uint ms )
 {
-    if ( xine_get_param( m_stream, XINE_PARAM_SPEED ) )
-        xine_play( m_stream, 0, (int)ms );
-    else {
-        // It was paused, then should keep paused
+    if( xine_get_param( m_stream, XINE_PARAM_SPEED ) == XINE_SPEED_PAUSE ) {
+        // FIXME this is a xine API issue really, they need to add a seek function
         xine_play( m_stream, 0, (int)ms );
         xine_set_param( m_stream, XINE_PARAM_SPEED, XINE_SPEED_PAUSE );
     }
+    else
+        xine_play( m_stream, 0, (int)ms );
 }
 
 void
 XineEngine::setVolumeSW( uint vol )
 {
-    xine_set_param( m_stream, XINE_PARAM_AUDIO_AMP_LEVEL, vol );
+    if( !s_fader )
+        xine_set_param( m_stream, XINE_PARAM_AUDIO_AMP_LEVEL, static_cast<uint>( vol * m_preamp ) );
 }
 
 void
 XineEngine::setEqualizerEnabled( bool enable )
 {
-   if( !enable ) {
-      QValueList<int> gains;
-      for( uint x = 0; x < 10; x++ )
-         gains += 0;
-      setEqualizerParameters( 0, gains );
+    m_equalizerEnabled = enable;
+
+    if( !enable ) {
+        QValueList<int> gains;
+        for( uint x = 0; x < 10; x++ )
+            gains << 0;
+
+        setEqualizerParameters( -100, gains );
    }
 }
 
 void
-XineEngine::setEqualizerParameters( int /*preamp*/, const QValueList<int> &gains )
+XineEngine::setEqualizerParameters( int preamp, const QValueList<int> &gains )
 {
-   QValueList<int>::ConstIterator it = gains.begin();
+    m_equalizerGains = gains;
+    m_intPreamp = preamp;
+    QValueList<int>::ConstIterator it = gains.begin();
+    debug() << "eq setting " << *it << endl;
+    xine_set_param( m_stream, XINE_PARAM_EQ_30HZ, *it );
+    xine_set_param( m_stream, XINE_PARAM_EQ_60HZ, *++it );
+    debug() << "eq setting " << *it << endl;
+    xine_set_param( m_stream, XINE_PARAM_EQ_125HZ, *++it );
+    debug() << "eq setting " << *it << endl;
+    xine_set_param( m_stream, XINE_PARAM_EQ_250HZ, *++it );
+    debug() << "eq setting " << *it << endl;
+    xine_set_param( m_stream, XINE_PARAM_EQ_500HZ, *++it );
+    debug() << "eq setting " << *it << endl;
+    xine_set_param( m_stream, XINE_PARAM_EQ_1000HZ, *++it );
+    debug() << "eq setting " << *it << endl;
+    xine_set_param( m_stream, XINE_PARAM_EQ_2000HZ, *++it );
+    debug() << "eq setting " << *it << endl;
+    xine_set_param( m_stream, XINE_PARAM_EQ_4000HZ, *++it );
+    debug() << "eq setting " << *it << endl;
+    xine_set_param( m_stream, XINE_PARAM_EQ_8000HZ, *++it );
+    debug() << "eq setting " << *it << endl;
+    xine_set_param( m_stream, XINE_PARAM_EQ_16000HZ, *++it );
+    debug() << "eq setting " << *it << endl;
 
-   xine_set_param( m_stream, XINE_PARAM_EQ_30HZ, *it );
-   xine_set_param( m_stream, XINE_PARAM_EQ_60HZ, *++it );
-   xine_set_param( m_stream, XINE_PARAM_EQ_125HZ, *++it );
-   xine_set_param( m_stream, XINE_PARAM_EQ_250HZ, *++it );
-   xine_set_param( m_stream, XINE_PARAM_EQ_500HZ, *++it );
-   xine_set_param( m_stream, XINE_PARAM_EQ_1000HZ, *++it );
-   xine_set_param( m_stream, XINE_PARAM_EQ_2000HZ, *++it );
-   xine_set_param( m_stream, XINE_PARAM_EQ_4000HZ, *++it );
-   xine_set_param( m_stream, XINE_PARAM_EQ_8000HZ, *++it );
-   xine_set_param( m_stream, XINE_PARAM_EQ_16000HZ, *++it );
+    m_preamp = ( preamp / 2 + 150 ) / 100.0;
+    setVolume( m_volume );
 }
 
 bool
@@ -453,8 +496,8 @@ XineEngine::timerEvent( QTimerEvent* )
    MyNode const * const list_end = myList;
 
    m_currentVpts = (xine_get_status( m_stream ) == XINE_STATUS_PLAY)
-      ? xine_get_current_vpts( m_stream )
-      : LLONG_MAX; //if state is not playing OR paused, empty the list
+         ? xine_get_current_vpts( m_stream )
+         : LLONG_MAX; //if state is not playing OR paused, empty the list
    //: std::numeric_limits<int64_t>::max(); //TODO don't support crappy gcc 2.95
 
    for( MyNode *prev = first_node, *node = first_node->next; node != list_end; node = node->next )
@@ -477,12 +520,17 @@ XineEngine::timerEvent( QTimerEvent* )
 amaroK::PluginConfig*
 XineEngine::configure() const
 {
-    return new XineConfigDialog( m_xine );
+    XineConfigDialog* xcf = new XineConfigDialog( m_xine );
+    connect(xcf, SIGNAL( settingsSaved() ), this, SLOT( configChanged() ));
+    connect(this, SIGNAL( resetConfig(xine_t*) ), xcf, SLOT( reset(xine_t*) ));
+    return xcf;
 }
 
 void
 XineEngine::customEvent( QCustomEvent *e )
 {
+    #define message static_cast<QString*>(e->data())
+
     switch( e->type() )
     {
     case 3000:
@@ -490,7 +538,6 @@ XineEngine::customEvent( QCustomEvent *e )
         break;
 
     case 3001:
-        #define message static_cast<QString*>(e->data())
         emit infoMessage( (*message).arg( m_url.prettyURL() ) );
         delete message;
         break;
@@ -498,7 +545,6 @@ XineEngine::customEvent( QCustomEvent *e )
     case 3002:
         emit statusText( *message );
         delete message;
-        #undef message
         break;
 
     case 3003: { //meta info has changed
@@ -509,18 +555,60 @@ XineEngine::customEvent( QCustomEvent *e )
         bundle.artist  = QString::fromUtf8( xine_get_meta_info( m_stream, XINE_META_INFO_ARTIST ) );
         bundle.album   = QString::fromUtf8( xine_get_meta_info( m_stream, XINE_META_INFO_ALBUM ) );
         bundle.comment = QString::fromUtf8( xine_get_meta_info( m_stream, XINE_META_INFO_COMMENT ) );
+        bundle.genre   = QString::fromUtf8( xine_get_meta_info( m_stream, XINE_META_INFO_GENRE ) );
+        bundle.bitrate = QString::number( xine_get_stream_info( m_stream, XINE_STREAM_INFO_BITRATE ) / 1000 );
 
         emit metaData( bundle );
-    }
+    }   break;
+
+    case 3004:
+        emit statusText( i18n("Redirecting to: ").arg( *message ) );
+        load( KURL( *message ), false );
+        play();
+        delete message;
+        break;
 
     default:
         ;
     }
+
+    #undef message
 }
+//SLOT
+void XineEngine::configChanged()
+{
+    //reset xine to load new audio plugin
+    if( m_currentAudioPlugin != XineCfg::outputPlugin() )
+    {
+        stop();
+        xine_config_save( m_xine, configPath() );
+        if( m_stream )     xine_close( m_stream );
+        if( m_eventQueue ) xine_event_dispose_queue( m_eventQueue );
+        m_eventQueue = NULL;
+        if( m_stream )     xine_dispose( m_stream );
+        m_stream = NULL;
+        if( m_audioPort )  xine_close_audio_driver( m_xine, m_audioPort );
+        m_audioPort = NULL;
+        if( m_post )       xine_post_dispose( m_xine, m_post );
+        m_post = NULL;
+        if( m_xine )       xine_exit( m_xine );
+        m_xine = NULL;
+        init();
+        setEqualizerEnabled( m_equalizerEnabled );
+        if( m_equalizerEnabled )
+                           setEqualizerParameters( m_intPreamp, m_equalizerGains );
+        emit resetConfig(m_xine);
+    }
+}
+
+static time_t last_error_time = 0; // hysteresis on xine errors
+static int    last_error = XINE_MSG_NO_ERROR;
 
 void
 XineEngine::XineEventListener( void *p, const xine_event_t* xineEvent )
 {
+    time_t current;
+
     if( !p ) return;
 
     #define xe static_cast<XineEngine*>(p)
@@ -541,13 +629,12 @@ XineEngine::XineEventListener( void *p, const xine_event_t* xineEvent )
         QApplication::postEvent( xe, new QCustomEvent(3000) );
         break;
 
-    case XINE_EVENT_PROGRESS:
-    {
+    case XINE_EVENT_PROGRESS: {
         xine_progress_data_t* pd = (xine_progress_data_t*)xineEvent->data;
 
         QString
         msg = "%1 %2%";
-        msg = msg.arg( QString( pd->description ) )
+        msg = msg.arg( QString::fromUtf8( pd->description ) )
                  .arg( KGlobal::locale()->formatNumber( pd->percent, 0 ) );
 
         QCustomEvent *e = new QCustomEvent( 3002 );
@@ -555,8 +642,20 @@ XineEngine::XineEventListener( void *p, const xine_event_t* xineEvent )
 
         QApplication::postEvent( xe, e );
 
-        break;
-    }
+    }   break;
+
+    case XINE_EVENT_MRL_REFERENCE: {
+        /// xine has read the stream and found it actually links to something else
+        /// so we need to play that instead
+
+        QString message = QString::fromUtf8( static_cast<xine_mrl_reference_data_t*>(xineEvent->data)->mrl );
+        QCustomEvent *e = new QCustomEvent( 3004 );
+        e->setData( new QString( message ) );
+
+        QApplication::postEvent( xe, e );
+
+    }   break;
+
     case XINE_EVENT_UI_MESSAGE:
     {
         debug() << "message received from xine\n";
@@ -612,11 +711,21 @@ XineEngine::XineEventListener( void *p, const xine_event_t* xineEvent )
 
         explain:
 
-            if(data->explanation)
+            // Don't flood the user with error messages
+            if( (last_error_time + 10) > time( &current ) &&
+                   data->type == last_error )
+            {
+                last_error_time = current;
+                return;
+            }
+            last_error_time = current;
+            last_error = data->type;
+
+            if( data->explanation )
             {
                 message.prepend( "<b>" );
                 message += "</b>:<p>";
-                message += ((char *) data + data->explanation);
+                message += QString::fromUtf8( (char*)data + data->explanation );
             }
             else break; //if no explanation then why bother!
 
@@ -624,13 +733,23 @@ XineEngine::XineEventListener( void *p, const xine_event_t* xineEvent )
 
         param:
 
+            // Don't flood the user with error messages
+            if((last_error_time + 10) > time(&current) &&
+               data->type == last_error)
+            {
+                last_error_time = current;
+                return;
+            }
+            last_error_time = current;
+            last_error = data->type;
+
             message.prepend( "<p>" );
             message += "<p>";
 
             if(data->explanation)
             {
                 message += "xine parameters: <i>";
-                message += ((char *) data + data->parameters);
+                message += QString::fromUtf8( (char*)data + data->parameters );
                 message += "</i>";
             }
             else message += i18n("Sorry, no additional information is available.");
@@ -651,6 +770,7 @@ XineEngine::XineEventListener( void *p, const xine_event_t* xineEvent )
 Fader::Fader( XineEngine *engine )
    : QObject( engine )
    , QThread()
+   , m_engine( engine )
    , m_xine( engine->m_xine )
    , m_decrease( engine->m_stream )
    , m_increase( 0 )
@@ -661,7 +781,6 @@ Fader::Fader( XineEngine *engine )
     {
         m_increase = engine->m_stream;
 
-        xine_set_param( m_decrease, XINE_PARAM_AUDIO_AMP_LEVEL, 100 );
         xine_set_param( m_increase, XINE_PARAM_AUDIO_AMP_LEVEL, 0 );
     }
     else {
@@ -674,12 +793,18 @@ Fader::~Fader()
 {
      wait();
 
-     debug() << k_funcinfo << endl;
+     DEBUG_FUNC_INFO
 
      xine_close( m_decrease );
      xine_dispose( m_decrease );
      xine_close_audio_driver( m_xine, m_port );
      if( m_post ) xine_post_dispose( m_xine, m_post );
+
+     if( !m_engine->m_stopFader )
+         m_engine->setVolume( m_engine->volume() );
+
+     m_engine->m_stopFader = false;
+     s_fader = 0;
 }
 
 struct fade_s {
@@ -703,10 +828,8 @@ Fader::run()
         //the usleep time for this volume
         sleeps[v] = int(120000.0 * (-log10( v+1 ) + 2));
 
-    for( int v = 99; v >= 0; --v ) {
+    for( int v = 99; v >= 0; --v )
         data.push_back( fade_s( sleeps[v], v, m_decrease ) );
-        kdDebug() << v << ": " << sleeps[v] << endl;
-    }
 
     {
         /**
@@ -714,14 +837,14 @@ Fader::run()
          * inbetween each volume increase/decrease
          */
 
-        int v = 0;
+        uint v = 0;
         int tu = 0;
         int td = sleeps[0];
         for( list<fade_s>::iterator it = data.begin(), end = data.end(); it != end; ++it ) {
             tu += (*it).sleep;
 
             while ( tu > td ) {
-                kdDebug() << tu << ", " << td << " for v=" << v << endl;
+//                 kdDebug() << tu << ", " << td << " for v=" << v << endl;
 
                 //this is the sleeptime for the structure we are about to insert
                 const int newsleep = tu - td;
@@ -733,31 +856,34 @@ Fader::run()
                 //insert the new structure for the increasing stream
                 data.insert( it, fade_s( newsleep, v, m_increase ) );
 
-                kdDebug() << "new: " << newsleep << endl;
+//                 kdDebug() << "new: " << newsleep << endl;
 
                 //decrease the contextual volume
-                if ( ++v > 99 )
+                if( ++v > 99 )
                     goto done;
 
                 //update td
                 td += sleeps[v];
             }
 
-            kdDebug() << tu << ", " << td << " for v=" << v << endl;
+//             kdDebug() << tu << ", " << td << " for v=" << v << endl;
         }
 
         done: ;
     }
 
     // perform the fading operations
-    for( list<fade_s>::iterator it = data.begin(), end = data.end(); it != end; ++it )
+    list<fade_s>::iterator it, end;
+    for( it = data.begin(), end = data.end(); it != end && !m_engine->m_stopFader; ++it )
     {
-        debug() << "sleep: " << (*it).sleep << " volume: " << (*it).volume << endl;
-
+//         debug() << "sleep: " << (*it).sleep << " volume: " << (*it).volume << endl;
         if( (*it).sleep > 0 ) //FIXME
            QThread::usleep( (*it).sleep );
 
-        xine_set_param( (*it).stream, XINE_PARAM_AUDIO_AMP_LEVEL, (*it).volume );
+        float vol = Engine::Base::makeVolumeLogarithmic( m_engine->m_volume );
+        vol = vol * m_engine->m_preamp * ( (*it).volume * 0.01 );
+
+        xine_set_param( (*it).stream, XINE_PARAM_AUDIO_AMP_LEVEL, (uint) vol );
     }
 
     //stop using cpu!
@@ -768,11 +894,4 @@ Fader::run()
     deleteLater();
 }
 
-
-namespace Debug
-{
-    #undef xine_indent
-    QCString xine_indent;
-}
-
 #include "xine-engine.moc"
