@@ -16,13 +16,15 @@
 #include "mediabrowser.h"
 
 #include <qdatetime.h>
+#include <qgroupbox.h>
 #include <qimage.h>
 #include <qlabel.h>
 #include <qpainter.h>
-#include <qpushbutton.h>
 #include <qregexp.h>
 #include <qsimplerichtext.h>
 #include <qtooltip.h>       //QToolTip::add()
+#include <qfileinfo.h>
+#include <qdir.h>
 
 #include <kapplication.h> //kapp
 #include <kdirlister.h>
@@ -32,8 +34,10 @@
 #include <kio/netaccess.h>
 #include <klocale.h>
 #include <kmessagebox.h>
+#include <kmountpoint.h>
 #include <kpopupmenu.h>
 #include <kprogress.h>
+#include <kpushbutton.h>
 #include <krun.h>
 #include <kstandarddirs.h> //locate file
 #include <ktabbar.h>
@@ -78,7 +82,6 @@ MediaBrowser::MediaBrowser( const char *name )
     setFocusProxy( m_view ); //default object to get focus
 }
 
-
 MediaBrowser::~MediaBrowser()
 {
     delete m_view;
@@ -297,7 +300,8 @@ MediaDeviceList::viewportPaintEvent( QPaintEvent *e )
         QSimpleRichText t( i18n(
                 "<div align=center>"
                   "<h3>MediaDevice Browser</h3>"
-                  "Drop files here to enqueue them for transfer to your iPod."
+                  "Click the Connect button to access your mounted iPod. "
+                  "Drag and drop files to enqueue them for transfer."
                 "</div>" ), QApplication::font() );
 
         t.setWidth( width() - 50 );
@@ -324,7 +328,7 @@ MediaDeviceList::rmbPressed( QListViewItem* item, const QPoint& point, int ) //S
                        BURN_DATACD, BURN_AUDIOCD, DELETE };
 
         menu.insertItem( SmallIconSet( "1downarrow" ), i18n( "&Append to Playlist" ), APPEND );
-        menu.insertItem( SmallIconSet( "2rightarrow" ), i18n( "&Queue After Current Track" ), QUEUE );
+        menu.insertItem( SmallIconSet( "2rightarrow" ), i18n( "&Queue Track" ), QUEUE );
         menu.insertItem( SmallIconSet( "player_playlist_2" ), i18n( "&Make Playlist" ), MAKE );
         menu.insertSeparator();
 
@@ -401,16 +405,30 @@ MediaDeviceView::MediaDeviceView( MediaBrowser* parent )
     m_transferList->setAcceptDrops( true );
     m_transferList->addColumn( i18n( "URL" ) );
 
-    QHBox* hb( this );
-    hb->setSpacing( 4 );
-    m_stats = new QLabel( i18n( "1 track in queue", "%n tracks in queue", m_transferList->childCount() ), hb );
-    m_progress = new KProgress( hb );
-    m_transferButton = new QPushButton( SmallIconSet( "rebuild" ), i18n( "Transfer" ), hb );
+    m_stats = new QLabel( i18n( "1 track in queue", "%n tracks in queue", m_transferList->childCount() ), this );
+    m_progress = new KProgress( this );
+
+    QHBox* hb = new QHBox( this );
+    hb->setSpacing( 1 );
+    m_connectButton  = new KPushButton( SmallIconSet( "usbpendrive_mount" ), i18n( "Connect"), hb );
+    m_transferButton = new KPushButton( SmallIconSet( "rebuild" ), i18n( "Transfer" ), hb );
+    m_configButton   = new KPushButton( KGuiItem( QString::null, "configure" ), hb );
+    m_configButton->setSizePolicy( QSizePolicy::Maximum, QSizePolicy::Preferred ); // too big!
+
+    QToolTip::add( m_connectButton,  i18n( "Connect media device" ) );
+    QToolTip::add( m_transferButton, i18n( "Transfer tracks to media device" ) );
+    QToolTip::add( m_configButton,   i18n( "Configure mount commands" ) );
+
+    m_connectButton->setToggleButton( true );
+    m_connectButton->setOn( m_deviceList->childCount() != 0 );
+    m_transferButton->setDisabled( true );
 
     m_progress->setFixedHeight( m_transferButton->sizeHint().height() );
     m_progress->hide();
 
+    connect( m_connectButton,  SIGNAL( clicked() ), MediaDevice::instance(), SLOT( ipodConnection() ) );
     connect( m_transferButton, SIGNAL( clicked() ), MediaDevice::instance(), SLOT( transferFiles() ) );
+    connect( m_configButton,   SIGNAL( clicked() ), MediaDevice::instance(), SLOT( config() ) );
 }
 
 
@@ -427,10 +445,42 @@ MediaDevice::MediaDevice( MediaDeviceView* parent )
 {
     s_instance = this;
 
+    sysProc = new KShellProcess(); Q_CHECK_PTR(sysProc);
+
     m_ipod = new IPod::IPod();
-    m_ipod->open( "/mnt/ipod" );
+
+    openIPod();
+
+    m_mntcmd = AmarokConfig::mountCommand();
+    m_umntcmd = AmarokConfig::umountCommand();
 }
 
+void
+MediaDevice::openIPod()
+{
+    if ( !m_ipod->isOpen() )
+        m_ipod->open( "/mnt/ipod" );
+
+    if ( !m_ipod->isOpen() )
+        m_ipod->open( "/media/iPod" ); // default path on SuSE 9.3
+
+    if ( !m_ipod->isOpen() ) {
+        // try to find a mounted ipod
+        KMountPoint::List currentmountpoints = KMountPoint::currentMountPoints();
+        KMountPoint::List::Iterator mountiter = currentmountpoints.begin();
+        for(; mountiter != currentmountpoints.end(); ++mountiter) {
+            QString mountpoint = (*mountiter)->mountPoint();
+            QString device = (*mountiter)->mountedFrom();
+
+            // only care about scsi devices (/dev/sd at the beginning or scsi somewhere in its name)
+            if (device.find("/dev/sd") != 0 && device.find("scsi") < 0)
+                continue;
+
+            if (m_ipod->open(mountpoint))
+                break;
+        }
+    }
+}
 
 MediaDevice::~MediaDevice()
 {
@@ -438,7 +488,6 @@ MediaDevice::~MediaDevice()
     delete m_ipod;
 }
 
-
 void
 MediaDevice::addURL( const KURL& url )
 {
@@ -453,10 +502,18 @@ MediaDevice::addURL( const KURL& url )
 
         m_transferURLs << url;
         m_parent->m_stats->setText( i18n( "1 track in queue", "%n tracks in queue", m_parent->m_transferList->childCount() ) );
+        m_parent->m_transferButton->setDisabled( false );
     } else
         amaroK::StatusBar::instance()->longMessage( i18n( "Track already exists on iPod: " + url.path().local8Bit() ) );
 }
 
+void
+MediaDevice::addURLs( const KURL::List urls )
+{
+        KURL::List::ConstIterator it = urls.begin();
+        for ( ; it != urls.end(); ++it )
+            addURL( (*it).path() );
+}
 
 QStringList
 MediaDevice::items( QListViewItem* item )
@@ -548,6 +605,80 @@ MediaDevice::songsByArtistAlbum( const QString& artist, const QString& album )
     return items;
 }
 
+void
+MediaDevice::config()
+{
+    KDialogBase dialog( m_parent, 0, false );
+    kapp->setTopWidget( &dialog );
+    dialog.setCaption( kapp->makeStdCaption( i18n("Configure Media Device") ) );
+    dialog.showButtonApply( false );
+    QVBox *box = dialog.makeVBoxMainWidget();
+
+    QLabel *mntLabel = new QLabel( box );
+    mntLabel->setText( i18n( "&Mount command:" ) );
+    QLineEdit *mntcmd = new QLineEdit( m_mntcmd, box );
+    mntLabel->setBuddy( mntcmd );
+    QToolTip::add( mntcmd, i18n( "Set the command to mount your device here, empty commands are not executed." ) );
+
+    QLabel *umntLabel = new QLabel( box );
+    umntLabel->setText( i18n( "&Unmount command:" ) );
+    QLineEdit *umntcmd = new QLineEdit( m_umntcmd, box );
+    umntLabel->setBuddy( umntcmd );
+    QToolTip::add( umntcmd, i18n( "Set the command to unmount your device here, empty commands are not executed." ) );
+
+    if ( dialog.exec() != QDialog::Rejected )
+    {
+        setMountCommand( mntcmd->text() );
+        setUmountCommand( umntcmd->text() );
+    }
+}
+
+void MediaDevice::setMountCommand(const QString & mnt)
+{
+    AmarokConfig::setMountCommand( mnt );
+    m_mntcmd = mnt;             //Update for mount()
+}
+
+void MediaDevice::setUmountCommand(const QString & umnt)
+{
+    AmarokConfig::setUmountCommand( umnt );
+    m_umntcmd = umnt;        //Update for umount()
+}
+
+int MediaDevice::mount()
+{
+    debug() << "mounting" << endl;
+    QString cmdS=m_mntcmd;
+
+    debug() << "attempting mount with command: [" << cmdS << "]" << endl;
+    int e=sysCall(cmdS);
+    debug() << "mount-cmd: e=" << e << endl;
+    return e;
+}
+
+int MediaDevice::umount()
+{
+    debug() << "umounting" << endl;
+    QString cmdS=m_umntcmd;
+
+    debug() << "attempting umount with command: [" << cmdS << "]" << endl;
+    int e=sysCall(cmdS);
+    debug() << "umount-cmd: e=" << e << endl;
+
+    return e;
+}
+
+int MediaDevice::sysCall(const QString & command)
+{
+    if ( sysProc->isRunning() )  return -1;
+
+        sysProc->clearArguments();
+        (*sysProc) << command;
+        if (!sysProc->start( KProcess::Block, KProcess::AllOutput ))
+            kdFatal() << i18n("could not execute %1").arg(command.local8Bit().data()) << endl;
+
+    return (sysProc->exitStatus());
+}
 
 void
 MediaDevice::transferFiles()  //SLOT
@@ -575,6 +706,17 @@ MediaDevice::transferFiles()  //SLOT
                 track.setPath( track.getPath() + ".mp3" );
                 QString trackpath = m_ipod->getRealPath( track.getPath() );
 
+                // check if path exists and make the path if needed
+                QFileInfo finfo( trackpath );
+                QDir dir = finfo.dir();
+                if ( !dir.exists() )
+                {
+                    QDir parentdir = dir;
+                    parentdir.cdUp();
+                    if ( parentdir.exists() )
+                        dir.mkdir( finfo.dirPath() );
+                }
+
                 m_wait = true;
 
                 KIO::CopyJob *job = KIO::copy( *it, KURL( trackpath ), false );
@@ -605,7 +747,7 @@ MediaDevice::transferFiles()  //SLOT
     else
         debug() << "iPod inconsistent!" << endl;
 
-    m_parent->m_transferButton->setEnabled( true );
+    m_parent->m_transferButton->setEnabled( false );
 }
 
 
@@ -615,10 +757,12 @@ MediaDevice::deleteFiles( const KURL::List& urls )
     //NOTE we assume that currentItem is the main target
     int count  = urls.count();
     int button = KMessageBox::warningContinueCancel( m_parent->m_parent,
-                    i18n( "<p>You have selected %1 to be <b>irreversibly</b> deleted." )
-                        .arg( i18n( "1 file", "%n files", count ) ),
-                    QString::null,
-                    i18n("&Delete") );
+                                                     i18n( "<p>You have selected 1 file to be <b>irreversibly</b> deleted.",
+                                                           "<p>You have selected %n files to be <b>irreversibly</b> deleted.",
+                                                           count
+                                                         ),
+                                                     QString::null,
+                                                     KGuiItem(i18n("&Delete"),"editdelete") );
 
     if ( button == KMessageBox::Continue )
     {
@@ -701,6 +845,57 @@ MediaDevice::deleteFromIPod( MediaItem* item )
     }
 }
 
+void
+MediaDevice::ipodConnection() //SLOT
+{
+    if ( m_parent->m_connectButton->isOn() ){
+        if ( !m_mntcmd.isEmpty() )
+        {
+            mount();
+        }
+        m_ipod = new IPod::IPod();
+        openIPod();
+        m_parent->m_deviceList->renderView( 0 );
+
+        if( m_parent->m_deviceList->childCount() != 0 )
+        {
+            m_parent->m_connectButton->setOn( true );
+        }
+        else
+        {
+            KMessageBox::error( m_parent->m_parent,
+                i18n( "Could not find device, please mount it and try again." ),
+                i18n( "Media Device Browser" ) );
+            m_parent->m_connectButton->setOn( false );
+        }
+    }
+    else{
+    if ( m_parent->m_transferList->childCount() != 0 &&  m_ipod->isStillConnected() )
+        {
+            int button = KMessageBox::warningContinueCancel( m_parent->m_parent,
+                                                        i18n( "There are tracks queued for transfer."
+                                                          " Would you like to transfer them before disconnecting?"),
+                                                         i18n( "Media Device Browser" ),
+                                                         KGuiItem(i18n("&Transfer"),"rebuild") );
+
+          if ( button == KMessageBox::Continue )
+                transferFiles();
+      }
+        fileTransferFinished();
+        m_ipod->close();
+        QString text=i18n( "Your device is now in sync, please unmount it and disconnect now." );
+        if ( !m_umntcmd.isEmpty() )
+        {
+            umount();
+            text=i18n( "Your device is now in sync, you can disconnect now." );
+        }
+        m_ipod = new IPod::IPod();
+        m_parent->m_deviceList->renderView( 0 );
+        m_parent->m_connectButton->setOn( false );
+        KMessageBox::error( m_parent->m_parent,
+            text, i18n( "Media Device Browser" ) );
+    }
+}
 
 bool
 MediaDevice::fileExists( const MetaBundle& bundle )
@@ -739,6 +934,7 @@ MediaDevice::fileTransferFinished()  //SLOT
 
     m_parent->m_stats->setText( i18n( "1 track in queue", "%n tracks in queue", m_parent->m_transferList->childCount() ) );
     m_parent->m_progress->hide();
+    m_parent->m_transferButton->setDisabled( true );
 }
 
 
@@ -756,7 +952,7 @@ MediaDevice::syncIPod()  //SLOT
     delete m_ipod;
 
     m_ipod = new IPod::IPod();
-    m_ipod->open( "/mnt/ipod" );
+    openIPod();
 
     m_parent->m_deviceList->renderView( 0 );
 }
