@@ -7,21 +7,29 @@
  * osd.cpp:   Shows some text in a pretty way independent to the WM
  * begin:     Fre Sep 26 2003
  * copyright: (C) 2004 Christian Muehlhaeuser <chris@chris.de>
- *            (C) 2004 Seb Ruiz <seb100@optusnet.com.au>
+ *            (C) 2004 Seb Ruiz <me@sebruiz.net>
  *            (C) 2004, 2005 Max Howell
+ *            (C) 2005 Gábor Lehel <illissius@gmail.com>
  */
 
+#include "amarok.h"
 #include "amarokconfig.h"
-#include "collectiondb.h"    //for albumCover location
 #include "debug.h"
+#include "collectiondb.h"    //for albumCover location
+#include "osd.h"
+#include "playlist.h"        //if osdUsePlaylistColumns()
+#include "playlistitem.h"    //ditto
+
 #include <kapplication.h>
 #include <kpixmap.h>
 #include <kpixmapeffect.h>
-#include "osd.h"
+#include <kstandarddirs.h>   //locate
+
 #include <qbitmap.h>
 #include <qpainter.h>
 #include <qregexp.h>
 #include <qtimer.h>
+#include <qvaluevector.h>
 
 namespace ShadowEngine
 {
@@ -132,8 +140,14 @@ OSDWidget::determineMetrics( const uint M )
                 QMIN( rect.height(), m_cover.height() ),
                 QImage::ScaleMin ); //this will force us to be with our bounds
 
+        int shadowWidth = 0;
+        if( m_drawShadow && !m_scaledCover.hasAlpha() &&
+          ( m_scaledCover.width() > 22 || m_scaledCover.height() > 22 ) )
+            shadowWidth = static_cast<uint>( m_scaledCover.width() / 100.0 * 6.0 );
+
         const int widthIncludingImage = rect.width()
                 + m_scaledCover.width()
+                + shadowWidth
                 + M; //margin between text + image
 
         rect.setWidth( widthIncludingImage );
@@ -210,12 +224,7 @@ OSDWidget::render( const uint M, const QSize &size )
         shadowColor = v > 128 ? Qt::black : Qt::white;
     }
 
-    int align = Qt::AlignVCenter | WordBreak;
-    switch( m_alignment ) {
-        case Left:  align |= Qt::AlignLeft; break;
-        case Right: align |= Qt::AlignRight; break;
-        default:    align |= Qt::AlignHCenter; break;
-    }
+    int align = Qt::AlignCenter | WordBreak;
 
     m_buffer.resize( rect.size() );
     QPainter p( &m_buffer );
@@ -233,19 +242,38 @@ OSDWidget::render( const uint M, const QSize &size )
 
     rect.addCoords( M, M, -M, -M );
 
-    if( !m_scaledCover.isNull() ) {
+    if( !m_cover.isNull() ) {
         QRect r( rect );
         r.setTop( (size.height() - m_scaledCover.height()) / 2 );
         r.setSize( m_scaledCover.size() );
-        p.drawPixmap( r.topLeft(), m_scaledCover );
 
-        if( !m_scaledCover.hasAlpha() ) {
-            // don't draw a border for eg, the amaroK icon
-            r.addCoords( -1, -1, 1, 1 );
-            p.setPen( shadowColor );
-            p.drawRect( r );
+        if( !m_scaledCover.hasAlpha() && m_drawShadow &&
+          ( m_scaledCover.width() > 22 || m_scaledCover.height() > 22 ) ) {
+            // don't draw a shadow for eg, the amaroK icon
+            QImage shadow;
+            const uint shadowSize = static_cast<uint>( m_scaledCover.width() / 100.0 * 6.0 );
+
+            const QString folder = amaroK::saveLocation( "covershadow-cache/" );
+            const QString file = QString( "shadow_albumcover%1x%2.png" ).arg( m_scaledCover.width()  + shadowSize )
+                                                                        .arg( m_scaledCover.height() + shadowSize );
+            if ( QFile::exists( folder + file ) )
+                shadow.load( folder + file );
+            else {
+                shadow.load( locate( "data", "amarok/images/shadow_albumcover.png" ) );
+                shadow = shadow.smoothScale( m_scaledCover.width() + shadowSize, m_scaledCover.height() + shadowSize );
+                shadow.save( folder + file, "PNG" );
+            }
+
+            QPixmap target;
+            target.convertFromImage( shadow ); //FIXME slow
+            copyBlt( &target, 0, 0, &m_scaledCover );
+            m_scaledCover = target;
+            r.setTop( (size.height() - m_scaledCover.height()) / 2 );
+            r.setSize( m_scaledCover.size() );
         }
 
+        p.drawPixmap( r.topLeft(), m_scaledCover );
+
         rect.rLeft() += m_scaledCover.width() + M;
     }
 
@@ -281,7 +309,7 @@ OSDWidget::event( QEvent *e )
             unsetColors(); //use new palette's colours
         return true;
     case QEvent::Paint:
-        //bitBlt( this, 0, 0, &m_buffer );
+        bitBlt( this, 0, 0, &m_buffer );
         return true;
     default:
         return QWidget::event( e );
@@ -311,8 +339,6 @@ OSDWidget::setScreen( int screen )
 }
 
 
-
-
 //////  OSDPreviewWidget below /////////////////////
 
 #include <kcursor.h>
@@ -416,64 +442,122 @@ void OSDPreviewWidget::mouseMoveEvent( QMouseEvent *e )
 #include "metabundle.h"
 #include <qregexp.h>
 
+template<class T>
+class MyVector: public QValueVector<T> //fucking QValueVector doesn't have operator<<, wtf?
+{
+    public:
+    inline MyVector<T> &operator<< (const T &x)
+    {
+        append( x );
+        return *this;
+    }
+};
+
 void
 amaroK::OSD::show( const MetaBundle &bundle ) //slot
 {
-    QString text = AmarokConfig::osdText();
-    QStringList tags, tokens;
+    QString text = "";
+    if( MetaBundle() == bundle )
+        text = i18n( "No track playing" );
 
-    // we special case prettyTitle and put it first
-    // so that we handle things like streams better
-    tokens << "%artist - %title"
-           << "%artist" << "%album" << "%title" << "%genre"
-           << "%year" << "%track" << "%bitrate" << "%length"
-           << "%file" << "%comment"
-           << "%score";
-
-    tags   << bundle.prettyTitle()
-           << bundle.artist() << bundle.album() << bundle.title() << bundle.genre()
-           << bundle.year() << bundle.track() << bundle.prettyBitrate()
-           << (bundle.length() > 0 ? bundle.prettyLength() : QString::null) //ignore '-' and '?'
-           << bundle.url().fileName() << bundle.comment()
-           << QString::number( CollectionDB::instance()->getSongPercentage( bundle.url().path() ) );
-
-    for( QStringList::ConstIterator tok = tokens.begin(), end = tokens.end(), tag = tags.begin(); tok != end; ++tok, ++tag )
+    else
     {
-        if( (*tag).isEmpty() ) {
-            text.remove( QRegExp(QString("\\{[^}]*%1[^{]*\\}").arg( *tok )) );
-            text.remove( *tok ); //above only works if {} surround the token
+        MyVector<QString> tags;
+        tags << bundle.prettyTitle();
+        for( int i = 0; i < PlaylistItem::NUM_COLUMNS; ++i )
+            tags << bundle.infoByColumn( i, true ).stripWhiteSpace();
+
+        const int score = CollectionDB::instance()->getSongPercentage( bundle.url().path() );
+        if( score > 0 )
+            tags[PlaylistItem::Score+1] = QString::number( score );
+        tags[PlaylistItem::Playcount+1] = QString::number( CollectionDB::instance()
+                                                           ->getPlayCount( bundle.url().path() ) );
+        if( bundle.length() <= 0 )
+            tags[PlaylistItem::Length+1] = QString::null;
+
+        if( AmarokConfig::osdUsePlaylistColumns() )
+        {
+            QString tag;
+            MyVector<int> availableTags; //eg, ones that aren't empty
+            static const QValueList<int> parens = //display these in parentheses
+                QValueList<int>() << PlaylistItem::Playcount << PlaylistItem::Year   << PlaylistItem::Comment
+                                  << PlaylistItem::Genre     << PlaylistItem::Length << PlaylistItem::Bitrate
+                                  << PlaylistItem::Score;
+
+            for( int n = Playlist::instance()->visibleColumns(), i = 0, column; i < n; ++i )
+            {
+                column = Playlist::instance()->mapToLogicalColumn( i );
+                if( !tags.at( column + 1 ).isEmpty() )
+                    availableTags << column;
+            }
+
+            for( int n = availableTags.count(), i = 0; i < n; ++i )
+            {
+                const int column = availableTags.at( i );
+                QString append = ( i == 0 ) ? ""
+                               : ( n > 1 && i == n / 2 ) ? "\n"
+                               : ( parens.contains( column ) || parens.contains( availableTags.at( i - 1 ) ) ) ? " "
+                               : " - ";
+                append += ( parens.contains( column ) ? "(%1)" : "%1" );
+                text += append.arg( tags.at( column + 1 ) );
+            }
         }
         else
-            //NOTE leaves the {} braces
-            text.replace( *tok, *tag );
+        {
+            // we special case prettyTitle and put it first
+            // so that we handle things like streams better
+            // keep these 1:1 with the playlist's columns otherwise
+            static const MyVector<QString> tokens =
+                MyVector<QString>() << "%artist - %title" << "%file"      << "%title"      << "%artist"
+                                    << "%album"           << "%year"      << "%comment"    << "%genre"
+                                    << "%track"           << "%directory" << "%length"     << "%bitrate"
+                                    << "%score"           << "%type"      << "%playcount";
+
+            text = AmarokConfig::osdText();
+
+            for( MyVector<QString>::ConstIterator tok = tokens.begin(), end = tokens.end(), tag = tags.begin(); tok != end; ++tok, ++tag )
+            {
+                if( (*tok).isNull() )
+                    continue;
+                if( (*tag).isEmpty() ) {
+                    text.remove( QRegExp(QString("\\{[^}]*%1[^{]*\\}").arg( *tok )) );
+                    text.remove( *tok ); //above only works if {} surround the token
+                }
+                else
+                    //NOTE leaves the {} braces
+                    text.replace( *tok, *tag );
+            }
+
+            text.remove( '{' );
+            text.remove( '}' );
+            text.replace( QRegExp("\n{2,}"), "\n" ); //multiple \n characters may remain
+            text.replace( "&lt;",  "<" );
+            text.replace( "&gt;",  ">" );
+            text.replace( "&amp;", "&" ); //replace some common HTML type stuff
+
+            // KDE 3.3 rejects \n in the .kcfg file, and KConfig turns \n into \\n, so...
+            text.replace( "\\n", "\n" );
+        }
+
+        if ( AmarokConfig::osdCover() ) {
+            //avoid showing the generic cover.  we can overwrite this by passing an arg.
+            //get large cover for scaling if big cover needed
+            QString location = CollectionDB::instance()->albumImage( bundle, 0 );
+
+            if ( location.find( "nocover" ) != -1 )
+                setImage( amaroK::icon() );
+            else
+                setImage( location );
+        }
+
+        text = text.stripWhiteSpace();
     }
 
-    text.remove( '{' );
-    text.remove( '}' );
-    text.replace( QRegExp("\n{2,}"), "\n" ); //multiple \n characters may remain
-    text.replace( "&lt;",  "<" );
-    text.replace( "&gt;",  ">" );
-    text.replace( "&amp;", "&" ); //replace some common HTML type stuff
+    if( text.isEmpty() )
+        text = MetaBundle::prettyTitle( bundle.url().fileName() ).stripWhiteSpace();
 
-    // KDE 3.3 rejects \n in the .kcfg file, and KConfig turns \n into \\n, so...
-    text.replace( "\\n", "\n" );
-
-    if ( AmarokConfig::osdCover() ) {
-        //avoid showing the generic cover.  we can overwrite this by passing an arg.
-        //get large cover for scaling if big cover needed
-        QString location = CollectionDB::instance()->albumImage( bundle, 0 );
-
-        if ( location.find( "nocover" ) != -1 )
-            setImage( amaroK::icon() );
-        else
-            setImage( location );
-    }
-
-    text = text.stripWhiteSpace();
-
-    //TODO after the freeze
-//    if( text.isEmpty() )
-//        text = i1_8n( "No track playing" );
+    if( text.isEmpty() ) //still
+        text = i18n("No information available for this track");
 
     OSDWidget::show( text );
 }
@@ -532,8 +616,8 @@ amaroK::OSD::forceToggleOSD()
  *
  * You should have received a copy of the GNU Library General Public License
  * along with this library; see the file COPYING.LIB.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * the Free Software Foundation, Inc., 51 Franklin Steet, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 namespace ShadowEngine
