@@ -11,13 +11,13 @@
 #define DEBUG_PREFIX "PlaylistLoader"
 
 #include "amarok.h"
-#include "collectiondb.h"
 #include "debug.h"
+#include "playlist.h"
+#include "collectiondb.h"
 #include "enginecontroller.h"
 #include <kapplication.h>
 #include <kdirlister.h>
 #include <kurl.h>
-#include "playlist.h"
 #include "playlistitem.h"
 #include "playlistloader.h"
 #include <qfile.h>       //::loadPlaylist()
@@ -26,6 +26,7 @@
 #include <qstringlist.h>
 #include <qtextstream.h> //::loadPlaylist()
 #include "statusbar.h"
+#include <dcopref.h>
 
 
 typedef QValueList<QDomNode> NodeList;
@@ -43,6 +44,9 @@ UrlLoader::UrlLoader( const KURL::List &urls, QListViewItem *after, bool playFir
 {
     DEBUG_BLOCK
 
+    connect( this,                 SIGNAL( queueChanged( const PLItemList &, const PLItemList & ) ),
+             Playlist::instance(), SIGNAL( queueChanged( const PLItemList &, const PLItemList & ) ) );
+
     Playlist::instance()->lock(); // prevent user removing items as this could be bad
 
     amaroK::OverrideCursor cursor;
@@ -68,6 +72,28 @@ UrlLoader::UrlLoader( const KURL::List &urls, QListViewItem *after, bool playFir
                 m_URLs += url;
         }
 
+       else if( protocol == "media" ) {
+            // url looks like media:/device/path
+            DCOPRef mediamanager( "kded", "mediamanager" );
+            QString device = url.path( -1 ).mid( 1 ); // remove first slash
+            const int slash = device.find( '/' );
+            const QString filePath = device.mid( slash ); // extract relative path
+            device = device.left( slash ); // extract device
+            DCOPReply reply = mediamanager.call( "properties(QString)", device );
+
+            if( reply.isValid() ) {
+                const QStringList properties = reply;
+                // properties[6] is the mount point
+                KURL localUrl = KURL( properties[6] + filePath );
+
+                // add urls
+                if( QFileInfo( localUrl.path() ).isDir() )
+                    m_URLs += recurse( localUrl );
+                else
+                    m_URLs += localUrl;
+            }
+        }
+
         else if( PlaylistFile::isPlaylistFile( url ) ) {
             new RemotePlaylistFetcher( url, after, m_playFirstUrl );
             m_playFirstUrl = false;
@@ -76,6 +102,9 @@ UrlLoader::UrlLoader( const KURL::List &urls, QListViewItem *after, bool playFir
         else if( protocol == "fetchcover" )
             continue;
 
+        else if( protocol == "seek" )
+            continue;
+
         else if( protocol == "album" ) {
            // url looks like:   album:<artist_id> @@@ <album_id>
            QString myUrl = url.path();
@@ -205,7 +234,7 @@ UrlLoader::customEvent( QCustomEvent *e )
     case 1001:
     {
         const QString QUEUE_INDEX = "queue_index";
-        const NodeList::ConstIterator end = e->nodes.end();
+        const QString STOP_AFTER = "stop_after";
         foreachType( NodeList, e->nodes )
         {
             if( (*it).isNull() ) //safety
@@ -225,7 +254,7 @@ UrlLoader::customEvent( QCustomEvent *e )
                     Playlist::instance()->setCurrentTrack( item );
 
                 else if( index > 0 ) {
-                    QPtrList<PlaylistItem> &m_nextTracks = Playlist::instance()->m_nextTracks;
+                    PLItemList &m_nextTracks = Playlist::instance()->m_nextTracks;
                     int count = m_nextTracks.count();
 
                     for( int c = count; c < index; c++ )
@@ -235,6 +264,8 @@ UrlLoader::customEvent( QCustomEvent *e )
                     m_nextTracks.replace( index - 1, item );
                 }
             }
+            if( element.hasAttribute( STOP_AFTER ) )
+                Playlist::instance()->m_stopAfterTrack = item;
         }
         break;
     }
@@ -251,8 +282,12 @@ UrlLoader::completeJob()
 {
     if ( !m_badURLs.isEmpty() ) {
         amaroK::StatusBar::instance()->shortLongMessage(
-                i18n("Some URLs were not suitable for the playlist."),
-                i18n("These URLs could not be loaded into the playlist: " ) );
+                i18n("Some media could not be loaded (not playable)."),
+                i18n("These media could not be loaded into the playlist: " ) );
+
+        debug() << "The following urls were not suitable for the playlist:" << endl;
+        for ( uint it = 0; it < m_badURLs.count(); it++  )
+            debug() << "\t" << m_badURLs[it] << endl;
     }
 
     // delete now or the playlist count in the statusbar will be wrong
@@ -316,24 +351,34 @@ UrlLoader::loadXml( const KURL &url )
         return;
     }
 
+    const PLItemList oldQueue = Playlist::instance()->m_nextTracks;
     NodeList nodes;
-    TagsEvent *e = new TagsEvent;
+    TagsEvent e;
     const QString ITEM( "item" ); //so we don't construct this QString all the time
     for( QDomNode n = d.namedItem( "playlist" ).firstChild(); !n.isNull(); n = n.nextSibling() )
     {
         if( n.nodeName() != ITEM ) continue;
 
         if( !n.toElement().isNull() )
-            e->nodes += n;
+            e.nodes += n;
 
-        if( e->nodes.count() == OPTIMUM_BUNDLE_COUNT ) {
-            QApplication::postEvent( this, e );
-            e = new TagsEvent;
+        if( e.nodes.count() == OPTIMUM_BUNDLE_COUNT ) {
+            QApplication::sendEvent( this, &e );
+            e = TagsEvent();
         }
     }
+    QApplication::sendEvent( this, &e );
 
-    QApplication::postEvent( this, e );
-};
+    const PLItemList &newQueue = Playlist::instance()->m_nextTracks;
+    QPtrListIterator<PlaylistItem> it( newQueue );
+    PLItemList added;
+    for( it.toFirst(); *it; ++it )
+        if( !oldQueue.containsRef( *it ) )
+            added << (*it);
+
+    if( !added.isEmpty() )
+        emit queueChanged( added, PLItemList() );
+}
 
 
 
@@ -549,7 +594,7 @@ PlaylistFile::loadRealAudioRam( QTextStream &stream )
     //while loop adapted from Kaffeine 0.5
     while (!stream.atEnd())
     {
-        url = stream.readLine(); 
+        url = stream.readLine();
         if (url[0] == '#') continue; /* ignore comments */
         if (url == "--stop--") break; /* stop line */
         if ((url.left(7) == "rtsp://") || (url.left(6) == "pnm://") || (url.left(7) == "http://"))
@@ -559,7 +604,7 @@ PlaylistFile::loadRealAudioRam( QTextStream &stream )
             b = MetaBundle();
         }
     }
-    
+
     return true;
 }
 /// @class RemotePlaylistFetcher
