@@ -6,11 +6,11 @@
 
 #include "amarok.h"
 #include "amarokconfig.h"
+#include "debug.h"
+#include "playlist.h"
 #include "collectiondb.h"
 #include "config.h"
-#include "debug.h"
 #include "enginecontroller.h"
-#include "playlist.h"
 #include "scrobbler.h"
 #include "statusbar.h"
 
@@ -41,7 +41,6 @@ Scrobbler* Scrobbler::instance()
 
 Scrobbler::Scrobbler() :
     EngineObserver( EngineController::instance() ),
-    m_prevPos( 0 ),
     m_validForSending( true ),
     m_submitter( new ScrobblerSubmitter() ),
     m_item( NULL )
@@ -117,7 +116,7 @@ void Scrobbler::similarArtists( const QString & artist )
         QDataStream stream( postData, IO_WriteOnly );
         stream.writeRawBytes( xmlRequest.utf8(), xmlRequest.length() );
 
-        m_similarArtistsBuffer = "";
+        m_similarArtistsBuffer = QString::null;
         m_artist = artist;
 
         KIO::TransferJob* job = KIO::http_post( "http://ws.audioscrobbler.com/xmlrpc", postData, false );
@@ -186,7 +185,7 @@ void Scrobbler::audioScrobblerSimilarArtistsResult( KIO::Job* job ) //SLOT
     }
 
     debug() << "Suggestions retrieved (" << suggestions.count() << ")" << endl;
-    if ( suggestions.count() > 0 )
+    if ( !suggestions.isEmpty() )
         emit similarArtistsFetched( m_artist, suggestions );
 }
 
@@ -211,6 +210,7 @@ void Scrobbler::engineNewMetaData( const MetaBundle& bundle, bool trackChanged )
         // Tags were changed, update them if not yet submitted.
         // TODO: In this case submit could be enabled if the artist or title
         // tag was missing initially and disabled submit
+        debug() << "It's still the same track." << endl;
         if ( m_item != NULL )
         {
             m_item->setArtist( bundle.artist() );
@@ -220,12 +220,12 @@ void Scrobbler::engineNewMetaData( const MetaBundle& bundle, bool trackChanged )
         return;
     }
 
-    m_prevPos = 0;
-
     // Plugins must not submit tracks played from online radio stations, even
     // if they appear to be providing correct metadata.
-    if ( bundle.streamUrl() != NULL )
+    if ( bundle.streamUrl() != NULL ) {
+        debug() << "Won't submit: It's a stream." << endl;
         m_validForSending = false;
+    }
     else
     {
         if ( m_item != NULL )
@@ -242,6 +242,7 @@ void Scrobbler::engineNewMetaData( const MetaBundle& bundle, bool trackChanged )
         {
             m_item = NULL;
             m_validForSending = false;
+            debug() << "Won't submit: No artist, no title, or less than 30 seconds." << endl;;
         }
     }
 }
@@ -250,25 +251,16 @@ void Scrobbler::engineNewMetaData( const MetaBundle& bundle, bool trackChanged )
 /**
  * Called when the signal is received.
  */
-void Scrobbler::engineTrackPositionChanged( long position )
+void Scrobbler::engineTrackPositionChanged( long position, bool userSeek )
 {
     if ( !m_validForSending )
         return;
 
-    long posChange = position - m_prevPos;
-    // If this is not the first position changed signal for this song.
-    if ( m_prevPos != 0 )
+    if ( userSeek )
     {
-        // TODO: It would be nice to have some more accurate method for
-        // detecting user seek. Now use MAIN_TIMER events with 2 sec
-        // tolerance.
-        if ( posChange > 2000 + EngineController::MAIN_TIMER )
-        {
-            // Position has changed more than it would during normal
-            // playback.
-            m_validForSending = false;
-            return;
-        }
+        m_validForSending = false;
+        debug() << "Won't submit: Seek detected." << endl;
+        return;
     }
 
     // Each track must be submitted to the server when it is 50% or 240
@@ -276,13 +268,9 @@ void Scrobbler::engineTrackPositionChanged( long position )
     if ( position > 240 * 1000 || position > 0.5 * m_item->length() * 1000 )
     {
         m_submitter->submitItem( m_item );
-        if ( AmarokConfig::dynamicMode() )
-            appendSimilar( m_item );
         m_item = NULL;
         m_validForSending = false;
     }
-
-    m_prevPos = position;
 }
 
 
@@ -310,24 +298,6 @@ void Scrobbler::applySettings()
 }
 
 
-/**
- * Appends suggested songs to playlist.
- */
-void Scrobbler::appendSimilar( SubmitItem* item ) const
-{
-    QStringList suggestions = CollectionDB::instance()->similarArtists( item->artist(), 16 );
-
-    QueryBuilder qb;
-    qb.setOptions( QueryBuilder::optRandomize | QueryBuilder::optRemoveDuplicates );
-    qb.addReturnValue( QueryBuilder::tabSong, QueryBuilder::valURL );
-    qb.addMatches( QueryBuilder::tabArtist, suggestions );
-    qb.setLimit( 0, 4 );
-    QStringList urls = qb.run();
-
-    Playlist::instance()->insertMedia( KURL::List( urls ), Playlist::Unique );
-}
-
-
 ////////////////////////////////////////////////////////////////////////////////
 // CLASS SubmitItem
 ////////////////////////////////////////////////////////////////////////////////
@@ -461,9 +431,9 @@ ScrobblerSubmitter::~ScrobblerSubmitter()
 {
     saveSubmitQueue();
 
-    m_ongoingSubmits.setAutoDelete( TRUE );
+    m_ongoingSubmits.setAutoDelete( true );
     m_ongoingSubmits.clear();
-    m_submitQueue.setAutoDelete( TRUE );
+    m_submitQueue.setAutoDelete( true );
     m_submitQueue.clear();
 }
 
@@ -539,11 +509,8 @@ void ScrobblerSubmitter::handshake()
     m_prevSubmitTime = currentTime;
     m_submitResultBuffer = "";
 
-    KIO::TransferJob* job = KIO::get( handshakeUrl, false, false );
-    connect( job, SIGNAL( result( KIO::Job* ) ),
-             this,  SLOT( audioScrobblerHandshakeResult( KIO::Job* ) ) );
-    connect( job, SIGNAL( data( KIO::Job*, const QByteArray& ) ),
-             this,  SLOT( audioScrobblerSubmitData( KIO::Job*, const QByteArray& ) ) );
+    KIO::TransferJob* job = KIO::storedGet( handshakeUrl, false, false );
+    connect( job, SIGNAL( result( KIO::Job* ) ), SLOT( audioScrobblerHandshakeResult( KIO::Job* ) ) );
 }
 
 
@@ -649,6 +616,9 @@ void ScrobblerSubmitter::submitItem( SubmitItem* item )
         if ( items[submitCounter] != 0 )
             m_ongoingSubmits.insert( job, items[submitCounter] );
 
+    amaroK::StatusBar::instance()->newProgressOperation( job )
+            .setDescription( i18n( "Submitting to last.fm" ) );
+
     connect( job, SIGNAL( result( KIO::Job* ) ),
              this,  SLOT( audioScrobblerSubmitResult( KIO::Job* ) ) );
     connect( job, SIGNAL( data( KIO::Job*, const QByteArray& ) ),
@@ -684,12 +654,12 @@ void ScrobblerSubmitter::setEnabled( bool enabled )
     if ( !enabled )
     {
         // If submit is disabled, clear submitqueue.
-        m_ongoingSubmits.setAutoDelete( TRUE );
+        m_ongoingSubmits.setAutoDelete( true );
         m_ongoingSubmits.clear();
-        m_ongoingSubmits.setAutoDelete( FALSE );
-        m_submitQueue.setAutoDelete( TRUE );
+        m_ongoingSubmits.setAutoDelete( false );
+        m_submitQueue.setAutoDelete( true );
         m_submitQueue.clear();
-        m_submitQueue.setAutoDelete( FALSE );
+        m_submitQueue.setAutoDelete( false );
     }
 }
 
@@ -699,12 +669,14 @@ void ScrobblerSubmitter::setEnabled( bool enabled )
  */
 void ScrobblerSubmitter::audioScrobblerHandshakeResult( KIO::Job* job ) //SLOT
 {
-    if ( job->error() )
-    {
+    if ( job->error() ) {
         warning() << "KIO error! errno: " << job->error() << endl;
         return;
     }
 
+    KIO::StoredTransferJob* const storedJob = static_cast<KIO::StoredTransferJob*>( job );
+    m_submitResultBuffer = QString::fromUtf8( storedJob->data().data(), storedJob->data().size() );
+
 //     debug()
 //         << "Handshake result received: "
 //         << endl << m_submitResultBuffer << endl;
@@ -771,8 +743,7 @@ void ScrobblerSubmitter::audioScrobblerHandshakeResult( KIO::Job* job ) //SLOT
  */
 void ScrobblerSubmitter::audioScrobblerSubmitResult( KIO::Job* job ) //SLOT
 {
-    if ( job->error() )
-    {
+    if ( job->error() ) {
         warning() << "KIO error! errno: " << job->error() << endl;
         enqueueJob( job );
         return;
@@ -959,10 +930,10 @@ void ScrobblerSubmitter::announceSubmit(
     if ( success )
     {
         if ( tracks == 1 )
-            _short = i18n( "'%1' submitted to Audioscrobbler" ).arg( item->title() );
+            _short = i18n( "'%1' submitted to last.fm" ).arg( item->title() );
         else
         {
-            _short = i18n( "Several tracks submitted to Audioscrobbler" );
+            _short = i18n( "Several tracks submitted to last.fm" );
 
             _long = "<p>";
             _long  = i18n( "'%1' and one other track submitted",
@@ -973,10 +944,10 @@ void ScrobblerSubmitter::announceSubmit(
     else
     {
         if ( tracks == 1 )
-            _short = i18n( "Failed to submit '%1' to Audioscrobbler" ).arg( item->title() );
+            _short = i18n( "Failed to submit '%1' to last.fm" ).arg( item->title() );
         else
         {
-            _short = i18n( "Failed to submit several tracks to Audioscrobbler" );
+            _short = i18n( "Failed to submit several tracks to last.fm" );
             _long  = "<p>";
             _long  = i18n( "Failed to submit '%1' and one other track",
                            "Failed to submit '%1' and %n other tracks", tracks-1 )
