@@ -22,6 +22,9 @@ class TagDialog : public TagDialogBase
     Q_OBJECT
 
     public:
+
+        enum Changes { NOCHANGE=0, SCORECHANGED=1, TAGSCHANGED=2 };
+
         TagDialog( const KURL& url, QWidget* parent = 0 );
         TagDialog( const KURL::List list, QWidget* parent = 0 );
         TagDialog( const MetaBundle& mb, PlaylistItem* item, QWidget* parent = 0 );
@@ -45,14 +48,17 @@ class TagDialog : public TagDialogBase
         void readMultipleTracks();
         void setMultipleTracksMode();
         bool hasChanged();
+        int changes();
         void storeTags();
         void saveTags();
         void saveMultipleTracks();
         bool writeTag( MetaBundle mb, bool updateCB=true );
 
         MetaBundle m_bundle;
+        int m_score;
         PlaylistItem* m_playlistItem;
         QMap<QString, MetaBundle> storedTags;
+        QMap<QString, int> storedScores;
         KURL::List m_urlList;
         QString m_buttonMbText;
         QString m_path;
